import srllib.qtgui
from PyQt4.QtCore import QObject, SIGNAL
import functools

def deferred_slot(func, optimize=False):
    """ Decorator for turning a method into a deferred slot.

    When calling a deferred slot, it is queued with the QApplication (must be
    a L{srllib.qtgui.Application} instance). Queued calls are dispatched
    periodically, which saves CPU time as opposed to making GUI calls directly
    as signals are received.
    """
    @functools.wraps(func)
    def schedule(*args, **kwds):
        srllib.qtgui.get_app().queue_deferred(func, args, kwds, optimize)
    return schedule

def deferred_slot_optimize(func):
    """ Optimized version of L{deferred_slot}.

    Optimization happens by only queueing one call to a slot at a time.
    """
    return deferred_slot(func, optimize=True)

class StatefulConnection(QObject):
    """ A connection between a Qt signal and a slot, which is capable of
    storing an extra set of arguments to the slot.

    We subclass QObject and make instances children of the signal emitter,
    so that their lifetime is bound to the latter.
    """
    def __init__(self, emitter, signal, slot, extra_args=[]):
        """
        @param emitter: The signal emitter.
        @param signal: Signal signature (PyQt4.QtCore.Signal is invoked on this).
        @param slot: The slot to be invoked.
        @param extra_args: Extra arguments to pass when invoking the slot.
        """
        QObject.__init__(self, emitter)
        self.__slot, self.__extra = slot, extra_args
        QObject.connect(emitter, SIGNAL(signal), self)

    def __call__(self, *args, **kwds):
        args = args + tuple(self.__extra)
        self.__slot(*args, **kwds)

def connect(emitter, signal, slot):
    """ Simplified version of QObject.connect which takes a raw slot signature.

    @param emitter: Signal emitter.
    @param signal: Signal signature (PyQt4.QtCore.Signal is invoked on this).
    @param slot: Signal signature (PyQt4.QtCore.Signal is invoked on this).
    """
    QObject.connect(emitter, SIGNAL(signal), slot)



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from testqtgui._common import *

import srllib.qtgui.widgets

class _LineEdit(srllib.qtgui.widgets._LineEditHelper, guimocks.QLineEditMock):
    _qbase = __qbase = guimocks.QLineEditMock

    def __init__(self, contents="", undo_stack=None, undo_text=None, pos=None):
        self.__qbase.__init__(self, returnValues={"text": contents})
        srllib.qtgui.widgets._LineEditHelper.__init__(self, undo_stack,
                undo_text, self.__qbase)
        if pos is None:
            pos = len(contents) + 1
        self.setCursorPosition(pos)

class LineEditTest(QtTestCase):
    def test_construct_with_undo(self):
        """ Test constructing with undo. """
        # Test default label for undo operation
        edit, stack = self.__construct("Test", undo=True)
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New")
        self.assertEqual(stack.undoText(), "edit text")

        # Test label for undo operation
        edit, stack = self.__construct("Test", undo=True, undo_text=
            "editing test")
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New")
        self.assertEqual(stack.undoText(), "editing test")

    def test_undo(self):
        """ Test undo functionality. """
        edit, stack = self.__construct("Initial", undo=True)
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New")
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New0")
        edit.emit(QtCore.SIGNAL("editingFinished()"))
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New1")
        stack.undo()
        edit.mockCheckNamedCall(self, "setText", -1, "New0")
        stack.undo()
        edit.mockCheckNamedCall(self, "setText", -1, "Initial")
        stack.redo()
        edit.mockCheckNamedCall(self, "setText", -1, "New0")
        stack.redo()
        edit.mockCheckNamedCall(self, "setText", -1, "New1")
        stack.undo()
        edit.mockCheckNamedCall(self, "setText", -1, "New0")

    def test_undo_setText(self):
        """ Test undo in conjunction with setText. """
        edit, stack = self.__construct(undo=True)
        edit.setText("Test")
        self.assertNot(stack.canUndo())
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "New")
        stack.undo()
        edit.mockCheckNamedCall(self, "setText", -1, "Test")

    def test_undo_setText_undoable(self):
        """ Test undo in conjunction with setText, with undoable=True. """
        edit, stack = self.__construct("Old", undo=True)
        edit.setText("New", undoable=True)
        stack.undo()
        edit.mockCheckNamedCall(self, "setText", -1, "Old")

    def test_editText_cursor(self):
        """Verify that the cursor position is kept."""
        edit, stack = self.__construct("Txt", undo=True, pos=1)
        edit.emit(QtCore.SIGNAL("textEdited(const QString&)"), "Text")
        self.assertEqual(edit.cursorPosition(), 1)

    def __construct(self, contents=QtCore.QString(), undo=False,
            undo_text=None, pos=None):
        if undo:
            undo_stack = QtGui.QUndoStack()
        edit = _LineEdit(contents, undo_stack=undo_stack, undo_text=undo_text,
                pos=pos)
        if not undo:
            return edit
        return edit, undo_stack

class _NumericalLineEdit(srllib.qtgui.widgets._NumericalLineEditHelper,
    _LineEdit):
    _qbase = _LineEdit

    def __init__(self, floating_point, contents, minimum, maximum):
        self._qbase.__init__(self, contents=contents)
        srllib.qtgui.widgets._NumericalLineEditHelper.__init__(self,
            floating_point, minimum, maximum)

class NumericalLineEditTest(QtTestCase):
    def test_construct(self):
        edit = self.__construct(True)
        call = edit.mockGetNamedCall("setValidator", 0)
        self.assert_(isinstance(call.args[0], QtGui.QDoubleValidator))
        edit = self.__construct(True, minimum=0, maximum=1)
        call = edit.mockGetNamedCall("setValidator", 0)
        vtor = call.args[0]
        self.assert_(isinstance(vtor, QtGui.QDoubleValidator))
        self.assertEqual(vtor.bottom(), 0)
        self.assertEqual(vtor.top(), 1)

        edit = self.__construct(False)
        call = edit.mockGetNamedCall("setValidator", 0)
        self.assert_(isinstance(call.args[0], QtGui.QIntValidator))
        edit = self.__construct(False, minimum=0, maximum=1)
        call = edit.mockGetNamedCall("setValidator", 0)
        vtor = call.args[0]
        self.assert_(isinstance(vtor, QtGui.QIntValidator))
        self.assertEqual(vtor.bottom(), 0)
        self.assertEqual(vtor.top(), 1)
        self.assertRaises(ValueError, self.__construct, False, minimum=0.1)
        self.assertRaises(ValueError, self.__construct, False, maximum=0.1)

    def __construct(self, floating_point, contents=QtCore.QString(),
        minimum=None, maximum=None):
        edit = _NumericalLineEdit(floating_point=
                floating_point, contents=contents, minimum=minimum, maximum=
                maximum)
        return edit

# Note that the helper must be inherited first, to override methods in the
# Qt base
class _CheckBox(srllib.qtgui.widgets._CheckBoxHelper, guimocks.QCheckBoxMock):
    _qbase = guimocks.QCheckBoxMock

    def __init__(self, undo_stack=None, undo_text=None):
        guimocks.QCheckBoxMock.__init__(self)
        srllib.qtgui.widgets._CheckBoxHelper.__init__(self, undo_stack=
            undo_stack, undo_text=undo_text)

class CheckBoxHelperTest(QtTestCase):
    def test_construct_with_undo(self):
        """ Test constructing with undo. """
        # Test default label for undo operation
        checkbox, stack = self.__construct(undo=True)
        self.__change_state(checkbox, True)
        self.assertEqual(stack.undoText(), "")

        # Test label for undo operation
        checkbox, stack = self.__construct(undo=True, undo_text=
            "check test")
        self.__change_state(checkbox, True)
        self.assertEqual(stack.undoText(), "check test")

    def test_undo(self):
        """ Test undo functionality. """
        checkbox, stack = self.__construct(undo=True)
        self.__change_state(checkbox, True)
        self.__change_state(checkbox, False)
        stack.undo()
        checkbox.mockCheckNamedCall(self, "setCheckState", -1, Qt.Checked)
        stack.undo()
        checkbox.mockCheckNamedCall(self, "setCheckState", -1, Qt.Unchecked)
        stack.redo()
        checkbox.mockCheckNamedCall(self, "setCheckState", -1, Qt.Checked)

    def __change_state(self, checkbox, checked):
        if checked:
            state = int(Qt.Checked)
        else:
            state = int(Qt.Unchecked)
        checkbox.emit(QtCore.SIGNAL("stateChanged(int)"), state)

    def __construct(self, checked=False, undo=False, undo_text=None):
        if undo:
            undo_stack = QtGui.QUndoStack()
        checkbox = _CheckBox(undo_stack=undo_stack, undo_text=undo_text)
        if not undo:
            return checkbox
        return checkbox, undo_stack



from PyQt4.QtCore import QObject

from srllib.testing.mock import *

class QMock(QObject, Mock):
    """ Mock class that also inherits from QObject.
    """
    __connections = {}

    def __init__(self, *args, **kwds):
        QObject.__init__(self)
        Mock.__init__(self, *args, **kwds)

    @classmethod
    def mock_clear_connections(cls):
        cls.__connections.clear()

    def mock_is_connected(self, slot, signal):
        return slot in self.__connections.setdefault(self, {}
            ).setdefault(signal, [])

    @classmethod
    def connect(cls, emitter, signal, slot):
        cls.__connections.setdefault(emitter, {}).setdefault(signal, []).append(
            slot)

    def emit(self, signal, *args):
        """ Allow signal emission.
        """
        # XXX: Consider slot signature?
        for slot in self.__connections.setdefault(self, {}).setdefault(
            signal, []):
            slot(*args)
        # QObject.emit(self, signal, *args)

class QWidgetMock(QMock):
    def __init__(self, *args, **kwds):
        QMock.__init__(self, *args, **kwds)
        self.__enabled = True

    def setEnabled(self, enabled):
        self.__enabled = enabled

    def isEnabled(self):
        return self.__enabled


class QDialogMock(QWidgetMock):
    pass



from _common import *
import srllib.qtgui

class BrowseFileButton(QToolButton):
    """ Standard button for browsing filesystem. """
    def __init__(self, parent=None, tooltip=None, icon=None):
        QToolButton.__init__(self, parent)
        if icon is not None:
            self.setIcon(icon)
        self.setText("...")
        if tooltip:
            self.setToolTip(tooltip)

class _BrowseHelper(object):
    def __init__(self, readonly=False, path=None, tooltip=None):
        if readonly:
            self.path_edit.setReadOnly(True)
        self.__tooltip = tooltip
        if path is not None:
            self.set_path(path)
        if tooltip:
            path_edit.setToolTip(tooltip)

    def set_path(self, path):
        """Set filepath."""
        self.path_edit.setText(path)

class _Browse(QWidget, _BrowseHelper):
    def __init__(self, parent, tooltip, browse_tooltip, readonly, path,
            icon):
        QWidget.__init__(self, parent)

        path_edit, browse_btn = self.path_edit, self.browse_button = \
                QLineEdit(self), BrowseFileButton(self, icon=icon)
        if browse_tooltip:
            browse_btn.setToolTip(browse_tooltip)

        lay = QHBoxLayout(self)
        lay.setMargin(0)
        lay.addWidget(path_edit, 1)
        lay.addWidget(browse_btn, 0, Qt.AlignRight)

        QObject.connect(browse_btn, SIGNAL("clicked()"), self.__slot_browse)

        _BrowseHelper.__init__(self, readonly, path, tooltip)

    def __slot_browse(self):
        fpath = self._get_filepath()
        if fpath is not None:
            self.set_path(QDir.toNativeSeparators(fpath))

class BrowseFile(_Browse):
    """ Widget composed of a QLineEdit and a L{BrowseFileButton} for browsing
    for a file.

    @cvar DefaultBrowseTooltip: Default tooltip for browse button.
    @ivar path_edit: QLineEdit for displaying/entering filepath.
    @ivar browse_button: L{BrowseFileButton} for opening browse dialog.
    """
    DefaultBrowseTooltip = "Browse for file"

    def __init__(self, parent=None, tooltip=None, browse_tooltip=DefaultBrowseTooltip,
            filter=None, readonly=False, path=None, icon=None):
        """
        @param tooltip: Optionally specify tooltip for line edit.
        @param browse_tooltip: Optionally specify tooltip for browse button.
        @param filter: Optionally specify a file filter (as a string, e.g.,
        "Images (*.png *.jpg)".
        @param readonly: Make line edit read-only?
        @param path: Optional initial path.
        @param icon: Optional button icon.
        """
        _Browse.__init__(self, parent, tooltip, browse_tooltip, readonly, path,
                icon)

        self.__filter = filter or QString()

    def _get_filepath(self):
        fpath = QFileDialog.getOpenFileName(self, "Open File", QString(),
                self.__filter)
        if not fpath.isNull():
            return fpath
        return None

class BrowseDirectory(_Browse):
    """ Widget composed of a QLineEdit and a L{BrowseFileButton} for browsing
    for a directory.

    @cvar DefaultBrowseTooltip: Default tooltip for browse button.
    @ivar path_edit: QLineEdit for displaying/entering filepath.
    @ivar browse_button: L{BrowseFileButton} for opening browse dialog.
    """
    DefaultBrowseTooltip = "Browse for directory"

    def __init__(self, parent=None, tooltip=None, browse_tooltip=DefaultBrowseTooltip,
            readonly=False, path=None, icon=None):
        """
        @param tooltip: Optionally specify tooltip for line edit.
        @param browse_tooltip: Optionally specify tooltip for browse button.
        @param readonly: Make line edit read-only?
        @param path: Optional initial path.
        @param icon: Optional button icon.
        """
        _Browse.__init__(self, parent, tooltip, browse_tooltip, readonly, path,
                icon)

    def _get_filepath(self):
        fpath = QFileDialog.getExistingDirectory(self, "Open Directory",
                self.path_edit.text())
        if not fpath.isNull():
            return fpath
        return None

class UndoStack(QtGui.QUndoStack):
    """ Specialization of QUndoStack).

    @ivar is_enabled: Enable pushing of commands? If disabled, commands are only
    performed, not pushed on the stack. Useful for modes when the undo stack
    shouldn't be updated.
    """
    __super = QtGui.QUndoStack

    def __init__(self, enable=True, parent=None):
        self.__super.__init__(self, parent)
        self.is_enabled = enable

    def push(self, command):
        if self.is_enabled:
            self.__super.push(self, command)
        else:
            command.redo()

def Action(text, slot=None, icon=None, shortcut=None, parent=None):
    """ Create a QAction.
    """
    if icon is not None:
        icon = QtGui.QIcon(icon)
    action = QtGui.QAction(icon, text, parent)
    if shortcut is not None:
        action.setShortcut(shortcut)
    if slot is not None:
        srllib.qtgui.connect(action, "triggered()", slot)
    return action



# -*- coding: utf-8 -*-
""" Test the util module. """
import os.path, stat, codecs

from srllib import util
import srllib.error as _srlerror

from _common import *

class FileSystemTest(TestCase):
    """ Test filesystem utility functions. """
    def test_movefile(self):

        # There might be a problem moving a file onto another, we are testing
        # this case as well since the destination already exists
        dstDir = self._get_tempdir()
        src, dst = self._get_tempfile(), util.create_file(os.path.join(dstDir, "test"))
        try: src.write("Test")
        finally: src.close()

        if util.get_os()[0] == util.Os_Windows:
            util.chmod(dst, 0)
        else:
            util.chmod(dstDir, 0)
        self.assertRaises(util.PermissionsError, util.move_file, src.name, dst)
        if util.get_os()[0] == util.Os_Windows:
            util.chmod(dst, 0700)
        else:
            util.chmod(dstDir, 0700)
        util.move_file(src.name, dst)
        f = file(dst)
        try: txt = f.read()
        finally: f.close()
        self.assertEqual(txt, "Test")
        self.assertNot(os.path.exists(src.name))

    def test_remove_dir(self):
        dpath = self.__create_dir()
        self.assertRaises(util.DirNotEmpty, util.remove_dir, dpath, recurse=False)
        util.remove_dir(dpath, recurse=True)
        self.assertNot(os.path.exists(dpath))

        # Test removing an empty dir
        dpath = self._get_tempdir()
        util.remove_dir(dpath, recurse=False)
        self.assertNot(os.path.exists(dpath))

    def test_remove_dir_force(self):
        """Test removing directory with read-only contents."""
        dpath = self.__create_dir()
        # Make the directory and its contents read-only
        util.chmod(dpath, 0, recursive=True)
        util.remove_dir(dpath, force=True)
        self.assertNot(os.path.exists(dpath))

    def test_remove_dir_force_relative(self):
        """Test removing directory forcefully with relative path."""
        dpath = self.__create_dir()
        dpath_parent = os.path.dirname(dpath)
        os.chdir(dpath_parent)
        dpath_rel = util.replace_root(dpath, "", dpath_parent)
        # Make sure it's a pure relative path, i.e. it doesn't start with ./
        assert not dpath_rel.startswith(".")

        util.remove_dir(dpath_rel, force=True)
        self.assertNot(os.path.exists(dpath))

    @only_posix
    def test_remove_dir_force_ro_parent(self):
        """Test attempting to forcefully remove directory with read-only parent."""
        dpath = self.__create_dir()
        # Make read-only
        util.chmod(dpath, 0555)
        self.assertRaises(util.PermissionsError, util.remove_dir,
                os.path.join(dpath, "testdir"), force=True)


    def test_remove_dir_missing(self):
        """ Test removing a missing directory. """
        self.assertRaises(ValueError, util.remove_dir, "nosuchdir")

    @only_posix
    def test_remove_dir_symlinked_dir(self):
        """Test remove_dir when there's a symlinked subdirectory that's removed before the symlink.

        If there's the subdirectory 'a' and in the same directory a symlink
        pointing to it, named 'b', remove_dir may remove the directory first
        thus breaking the symlink. In this case, remove_dir may still think
        the 'b' is a directory since it originally pointed to one.
        """
        dpath = self._get_tempdir()
        os.mkdir(os.path.join(dpath, "a"))
        os.symlink("a", os.path.join(dpath, "b"))

        util.remove_dir(dpath)
        self.assertNot(os.path.exists(dpath))
        

    def test_copy_dir(self):
        """ Test copying a directory.

        When a directory is copied, the client callback should be called periodically with
        progress status.
        """
        def callback(progress):
            self.__progress.append(progress)

        dpath = self.__create_dir()
        dstDir = self._get_tempdir()
        self.assertRaises(util.DestinationExists, util.copy_dir, dpath, dstDir)

        # Try forcing deletion of existing directory

        os.mkdir(os.path.join(dstDir, "testdir"))
        # Remove only write permission
        if util.get_os()[0] == util.Os_Windows:
            util.chmod(os.path.join(dstDir, "testdir"), 0500)
        else:
            util.chmod(dstDir, 0500)
        # Make sure we can access the directory (requires the correct
        # permissions on dstDir)
        assert os.path.exists(os.path.join(dstDir, "testdir"))
        # This should fail, because the conflicting directory is protected from
        # deletion
        self.assertRaises(util.PermissionsError, util.copy_dir, dpath,
                os.path.join(dstDir, "testdir"), mode=util.CopyDir_Delete)
        if util.get_os()[0] == util.Os_Windows:
            util.chmod(os.path.join(dstDir, "testdir"), 0700)
        else:
            util.chmod(dstDir, 0700)
        util.copy_dir(dpath, dstDir, mode=util.CopyDir_Delete)

        util.remove_dir(dstDir, recurse=True)
        self.__progress = []
        util.copy_dir(dpath, dstDir, callback=callback)
        self.assertEqual(compare_dirs(dpath, dstDir), ([], []))
        self.assertEqual(self.__progress[0], 0.0)
        self.assertEqual(self.__progress[-1], 100.0)

        # Test ignoring certain files
        dpath = self._get_tempdir()
        os.mkdir(os.path.join(dpath, ".svn"))
        util.create_file(os.path.join(dpath, "test"))
        dstDir = self._get_tempdir()
        util.copy_dir(dpath, dstDir, ignore=[".*"], mode=util.CopyDir_Delete)
        self.assertEqual(os.listdir(dstDir), ["test"])

    def test_copy_dir_noperm(self):
        """ Test copying a directory with missing permissions. """
        dpath0, dpath1 = self.__create_dir(), self._get_tempdir()
        if util.get_os()[0] != util.Os_Windows:
            # Can't remove read permission on Windows
            util.chmod(dpath0, 0)
            self.assertRaises(util.PermissionsError, util.copy_dir, dpath0,
                    dpath1, mode=util.CopyDir_Delete)
            util.chmod(dpath0, 0700)
            # Test directory permissions
            util.chmod(os.path.join(dpath0, "testdir"), 0000)
            self.assertRaises(util.PermissionsError, util.copy_dir, dpath0,
                    dpath1, mode=util.CopyDir_Delete)
            util.chmod(os.path.join(dpath0, "testdir"), 0700)
            # Test file permissions
            util.chmod(os.path.join(dpath0, "test"), 0000)
            self.assertRaises(util.PermissionsError, util.copy_dir, dpath0,
                    dpath1, mode=util.CopyDir_Delete)

    def test_copy_dir_missing_source(self):
        """Test copy_dir with missing source directory."""
        src_dir = self.__create_dir()
        util.remove_dir(src_dir)
        dst_dir = self.__create_dir()
        self.assertRaises(util.MissingSource, util.copy_dir, src_dir,
                dst_dir, mode=util.CopyDir_Delete)

    def test_copy_dir_delete(self):
        """ Test copying directory after first deleting the destination. """
        srcdir = self.__create_dir()
        dstdir = self._get_tempdir()
        util.copy_dir(srcdir, dstdir, mode=util.CopyDir_Delete)
        self.assertEqual(os.listdir(dstdir), os.listdir(srcdir))

    def test_copy_dir_merge(self):
        """ Test copying directory while merging new contents with old. """
        srcdir = self._get_tempdir()
        dstdir = self._get_tempdir()
        # Make sure that the merge operation replaces directories in the
        # destination directory with files in the source directory and vice
        # versa
        os.mkdir(os.path.join(srcdir, "testdir"))
        util.create_file(os.path.join(srcdir, "testfile"), "Test")
        util.create_file(os.path.join(srcdir, "oldfile"), "Test")
        util.create_file(os.path.join(dstdir, "newfile"), "Test")
        util.create_file(os.path.join(dstdir, "testdir"), "Test")
        os.mkdir(os.path.join(dstdir, "testfile"))
        util.copy_dir(srcdir, dstdir, mode=util.CopyDir_Merge)

        self.assertSortedEqual(os.listdir(dstdir), ["testdir", "testfile",
            "oldfile", "newfile"])
        self.assert_(os.path.isdir(os.path.join(dstdir, "testdir")))
        for fname in ("testfile", "oldfile", "newfile"):
            self.assert_(os.path.isfile(os.path.join(dstdir, fname)))

    def test_copy_dir_writefile(self):
        """ Test passing a custom writefile function to copy_dir. """
        def copyfile(src, dst, callback, **kwds):
            self.__invoked = True
            callback(50.0)
            callback(100.0)
        def callback(progress):
            self.__progress.append(progress)

        srcdir, dstdir = self._get_tempdir(), self._get_tempdir()
        util.create_file(os.path.join(srcdir, "test1"), "Test")
        self.__invoked = False
        self.__progress = []
        util.copy_dir(srcdir, dstdir, mode=util.CopyDir_Delete, copyfile=
            copyfile, callback=callback)
        self.assert_(self.__invoked, "copyfile was not invoked")
        self.assertEqual(self.__progress, [0, 50.0, 100.0],
            "Wrong progress: %r" % self.__progress)

    if util.get_os_name() in util.OsCollection_Posix:
        def test_copy_dir_symlink(self):
            """ Test copying directory with symlink. """
            srcdir, dstdir = self._get_tempdir(), self._get_tempdir()
            util.create_file(os.path.join(srcdir, "test"), "Test")
            os.symlink("test", os.path.join(srcdir, "sym"))
            util.copy_dir(srcdir, dstdir, mode=util.CopyDir_Delete)
            self.assert_(os.path.islink(os.path.join(dstdir, "sym")),
                "Symlink dereferenced")
            self.assertEqual(os.readlink(os.path.join(dstdir, "sym")), "test",
                "Symlink not copied correctly")

    def test_copy_dir_fs_mode(self):
        """Test copy_dir with specific fs_mode argument."""
        srcdir, dstdir = self._get_tempdir(), self._get_tempdir()
        util.create_file(os.path.join(srcdir, "test"), "Test")

        if util.get_os_name() != Os_Windows:
            fs_mode = 0755
        else:
            # This is a mode that'll work well on Windows for files
            fs_mode = 0666
        util.copy_dir(srcdir, dstdir, mode=util.CopyDir_Delete,
                fs_mode=fs_mode)

        pathnames = [os.path.join(dstdir, "test")]
        if util.get_os_name() != Os_Windows:
            # It's very restricted which permissions we can set on Windows directories
            pathnames.append(dstdir)
        for pathname in pathnames:
            got_mode = stat.S_IMODE(os.lstat(pathname).st_mode)
            self.assertEqual(got_mode, fs_mode,
                    "Mode not correctly set on '%s' (%o)" % (pathname,
                        got_mode))

    def test_copy_file(self):
        src, dst = self._get_tempfile(), self._get_tempfname()
        try: src.write("Test")
        finally: src.close()
        util.copy_file(src.name, dst)
        f = file(dst)
        try: txt = f.read()
        finally: f.close()
        self.assertEqual(txt, "Test")

    def test_copy_file_missing(self):
        src, dst = self._get_tempfname(), self._get_tempfname()
        os.remove(src)
        self.assertRaises(util.MissingSource, util.copy_file, src, dst)

    def test_create_tempfile(self):
        fname = self.__create_tempfile()
        try: self.assert_(isinstance(fname, basestring))
        finally: os.remove(fname)
        file_ = self.__create_tempfile(close=False)
        try: self.assert_(isinstance(file_, file))
        finally:
            file_.close()
            os.remove(file_.name)
        file_ = self.__create_tempfile(close=False, content="Test")
        try:
            txt = file_.read()
            self.assertEqual(txt, "Test")
            # XXX: On Windows, we have to write from the beginning of file for
            # some reason, otherwise IOError is raised
            file_.seek(0)
            file_.write(txt + "\nTest")
            file_.seek(0)
            self.assertEqual(file_.read(), "Test\nTest")
        finally: file_.close()

    def test_create_tempfile_invalid_encoding(self):
        self.assertRaises(UnicodeDecodeError, self.__create_tempfile,
                content="æøå", encoding="ascii")

    def test_chmod(self):
        dpath = self.__create_dir()
        util.chmod(dpath, 0)
        mode = stat.S_IMODE(os.stat(dpath).st_mode)
        if util.get_os()[0] == util.Os_Windows:
            # Impossible to remove rx permissions on Windows
            self.assertNot(mode & stat.S_IWRITE)
        else:
            self.assertEqual(mode, 0)
            self.assertRaises(util.PermissionsError, util.chmod, dpath, 0,
                    recursive=True)
        util.chmod(dpath, 0700)
        util.chmod(dpath, 0000, recursive=True)
        util.chmod(dpath, 0700)
        filemode = stat.S_IMODE(os.stat(os.path.join(dpath, "test")).st_mode)
        dirmode = stat.S_IMODE(os.stat(os.path.join(dpath, "testdir")).st_mode)
        if util.get_os()[0] == util.Os_Windows:
            self.assertNot(filemode & (stat.S_IWRITE | stat.S_IEXEC))
            self.assertNot(dirmode & stat.S_IWRITE)
        else:
            self.assertEqual(filemode, 0)
            self.assertEqual(dirmode, 0)

    def test_chmod_recursive(self):
        """ Test chmod in recursive mode. """
        dpath = self._get_tempdir()
        fpath = util.create_file(os.path.join(dpath, "file"))
        os.mkdir(os.path.join(dpath, "subdir"))
        # Set executable so the directory can be traversed
        mode = stat.S_IREAD | stat.S_IEXEC
        util.chmod(dpath, mode, True)
        if util.get_os_name() == util.Os_Windows:
            # Some permissions can't be turned off on Windows ..
            dirmode = mode | (stat.S_IROTH | stat.S_IXOTH | stat.S_IRGRP | stat.S_IXGRP)
            fmode = stat.S_IREAD | stat.S_IROTH | stat.S_IRGRP
        else:
            dirmode = fmode = mode
        self.assertEqual(util.get_file_permissions(dpath), dirmode)
        for e in os.listdir(dpath):
            if os.path.isfile(os.path.join(dpath, e)):
                mode = fmode
            else:
                mode = dirmode
            self.assertEqual(util.get_file_permissions(os.path.join(dpath, e)),
                    mode)

    def test_walkdir(self):
        entered = []
        root = self.__create_dir()
        for dpath, dnames, fnames in util.walkdir(root):
            entered.append(dpath)
        self.assertEqual(entered, [root, os.path.join(root, "testdir")])

    def test_walkdir_ignore(self):
        """ Test ignoring files.
        """
        root = self.__create_dir()
        os.mkdir(os.path.join(root, ".svn"))
        entered = []
        for dpath, dnames, fnames in util.walkdir(root, ignore=[".svn"]):
            entered.append(dpath)
        self.assertEqual(entered, [root, os.path.join(root, "testdir")])

    def test_remove_file(self):
        dpath = self._get_tempdir()
        fpath = util.create_file(os.path.join(dpath, "test"))
        if util.get_os()[0] == util.Os_Windows:
            util.chmod(fpath, 0)
        else:
            util.chmod(dpath, 0)
        self.assertRaises(util.PermissionsError, util.remove_file, fpath)
        if util.get_os()[0] == util.Os_Windows:
            util.chmod(fpath, 0700)
        else:
            util.chmod(dpath, 0700)
        util.remove_file(fpath)
        self.assertNot(os.path.exists(fpath))

    def test_remove_file_force(self):
        """ Test removing a read-only file forcefully. """
        dpath = self.__create_dir()
        fpath = os.path.join(dpath, "test")
        # Make the file read-only
        util.chmod(fpath, stat.S_IEXEC | stat.S_IREAD, recursive=True)
        util.remove_file(fpath, force=True)
        self.assertNot(os.path.exists(fpath))

    def test_remove_file_or_dir(self):
        dpath, fpath = self.__create_dir(), self._get_tempfname()
        util.remove_file_or_dir(dpath, recurse=True)
        util.remove_file_or_dir(fpath)
        self.assertNot(os.path.exists(dpath))
        self.assertNot(os.path.exists(fpath))

    def test_get_os(self):
        self.assertIn(srllib.util.get_os()[0], (srllib.util.Os_Linux,
                srllib.util.Os_Windows, srllib.util.Os_Mac))

    def test_create_file_unicode(self):
        """ Test creating file with unicode content. """
        fpath = self._get_tempfname()
        util.create_file(fpath, content=u"æøå", encoding="utf-8")
        f = codecs.open(fpath, encoding="utf-8")
        try: self.assertEqual(f.read(), u"æøå")
        finally: f.close()

    def test_read_file_unicode(self):
        """ Test reading a file with unicode content. """
        fpath = self._get_tempfname()
        f = codecs.open(fpath, encoding="utf-8", mode="wb")
        try: f.write(u"Æøå")
        finally: f.close()
        self.assertEqual(util.read_file(fpath, encoding="utf-8"), u"Æøå")

    def test_create_file_bin(self):
        """ Test creating a file in binary mode. """
        f = util.create_file(self._get_tempfname(), binary=True, close=False)
        try:
            self.assertEqual(f.mode, "wb")
        finally: f.close()

    def test_create_file_invalid_encoding(self):
        """ Test creating a file with invalid encoding. """
        self.assertRaises(UnicodeDecodeError, util.create_file,
                self._get_tempfname(), content="æøå", encoding="ascii")

    def test_replace_root(self):
        """Test function replace_root."""
        fpath, newroot = self._get_tempfname(), self._get_tempdir()
        self.assertEqual(util.replace_root(fpath, newroot,
                os.path.dirname(fpath)), os.path.join(newroot,
                os.path.basename(fpath)))

    def test_replace_root_default(self):
        """ Test replace_root with default original root. """
        if util.get_os_name() in util.OsCollection_Posix:
            fpath, newroot = "/file", "/tmp/"
        elif util.get_os_name() == util.Os_Windows:
            fpath, newroot = r"C:\file", "Z:\\"
        self.assertEqual(util.replace_root(fpath, newroot), os.path.join(
                newroot, "file"))

    def test_replace_root_noroot(self):
        """ Test calling replace_root with a name with no directory component.
        """
        self.assertEqual(util.replace_root("file", "some root"), "file")

    def test_replace_root_badroot(self):
        """Test replace_root with an invalid original root."""
        self.assertRaises(ValueError, util.replace_root, os.path.join("root",
            "file"), "", "badroot")

    def test_replace_root_relative(self):
        """Test replace_root so the result is relative."""
        self.assertEqual(util.replace_root(os.path.join("dir", "file"), "",
            "dir"), "file")

    if util.get_os_name() == util.Os_Windows:
        def test_replace_root_unix_pathsep(self):
            """Test function replace_root with UNIX path separators in path (could cause bug on Windows)."""
            fpath = r"C:\thisdir/file"
            newroot = "C:/thatdir"
            oldroot = os.path.dirname(fpath)
            self.assertEqual(util.replace_root(fpath, newroot, oldroot),
                    r"C:\thatdir\file")


    def test_resolve_path(self):
        """ Test resolving path to executable. """
        self.assertEquals(os.path.splitext(os.path.basename(
                util.resolve_path("python")))[0], "python")

    if util.get_os_name() == util.Os_Windows:
        def test_resolve_path_with_ext(self):
            """Test resolving path to executable with extension in the name."""
            self.assertEquals(os.path.splitext(os.path.basename(
                    util.resolve_path("python.exe")))[0], "python")

    def test_resolve_path_notfound(self):
        """ Test resolving non-existent executable. """
        self.assertRaises(_srlerror.NotFound, util.resolve_path,
                "There is no such executable.")

    def test_resolve_path_badname(self):
        """ resolve_path should not accept a pathname with directory components.
        """
        self.assertRaises(ValueError, util.resolve_path, os.path.join("dir",
            "file"))

    def test_compare_dirs(self):
        """ Test dir comparison. """
        dpath0 = self._get_tempdir()
        util.create_file(os.path.join(dpath0, "tmpfile"), "Test")
        self.assertEqual(util.compare_dirs(dpath0, dpath0, False), ([], []))

    def test_compare_dirs_missing(self):
        """ Test supplying missing directories to compare_dirs.
        """
        dpath = self._get_tempdir()
        self.assertRaises(ValueError, util.compare_dirs, "nosuchdir", dpath)
        self.assertRaises(ValueError, util.compare_dirs, dpath, "nosuchdir")

    def test_compare_dirs_first_empty(self):
        """ Test against an empty first directory. """
        dpath0, dpath1 = self._get_tempdir(), self._get_tempdir()
        util.create_file(os.path.join(dpath1, "file"))
        self.assertEqual(util.compare_dirs(dpath0, dpath1), ([], ["file"]))

    def test_compare_dirs_subdirs(self):
        """ Test compare_dirs with differing sub-directories. """
        dpath0, dpath1 = self._get_tempdir(), self._get_tempdir()
        subdir0, subdir1 = os.path.join(dpath0, "subdir"), os.path.join(dpath1,
                 "subdir")
        os.mkdir(subdir0)
        os.mkdir(subdir1)
        util.chmod(subdir0, 0)
        self.assertEqual(util.compare_dirs(dpath0, dpath1), (["subdir"], []))

    def test_clean_path(self):
        self.assertEqual(util.clean_path(os.path.join("dir", "..", "file")),
                os.path.join(os.path.abspath("file")))

    def test_get_checksum_cancel(self):
        """ Test canceling checksum calculation. """
        def callback():
            raise _srlerror.Canceled

        path = util.create_file(self._get_tempfname(), "Test")
        self.assertRaises(_srlerror.Canceled, util.get_checksum, path, callback=
            callback)

    def __create_dir(self):
        """ Create directory with contents. """
        dpath = self._get_tempdir()
        # We put some content inside the created files, since read permissions
        # will not affect empty files (i.e., copying an empty file won't
        # provoke an error)
        util.create_file(os.path.join(dpath, "test"), "Test")
        os.mkdir(os.path.join(dpath, "testdir"))
        util.create_file(os.path.join(dpath, "testdir", "test"), "Test")
        return dpath

    def __create_tempfile(self, *args, **kwds):
        fpath = util.create_tempfile(*args, **kwds)
        self._tempfiles.append(fpath)
        return fpath

class VariousTest(TestCase):
    """ Test various functionality. """
    def test_get_checksum(self):
        dir0, dir1 = self._get_tempdir(), self._get_tempdir()
        self.assertEqual(util.get_checksum(dir0), util.get_checksum(dir1))
        fpath = util.create_file(os.path.join(dir1, "test"), "Test")
        chksum = util.get_checksum(dir1)
        self.assertNotEqual(util.get_checksum(dir0), chksum)
        self.assertEqual(util.get_checksum(fpath), chksum)

    def test_get_checksum_invalid_format(self):
        """ Pass invalid format to get_checksum. """
        self.assertRaises(ValueError, util.get_checksum, "somepath", -1)

    def test_get_checksum_bin(self):
        """ Test binary checksum (20 bytes). """
        self.assertEqual(len(util.get_checksum(self._get_tempfname(),
                util.Checksum_Binary)), 20)

    def test_get_module(self):
        """ Test the get_module function. """
        fpath = self._get_tempfname(content="test = True\n", suffix=".py")
        try: m = util.get_module(os.path.splitext(os.path.basename(fpath))[0],
                os.path.dirname(fpath))
        finally:
            # Remove .pyc
            os.remove(fpath + "c")
        self.assertEqual(m.test, True)

    def test_get_module_missing(self):
        """ Try finding a missing module. """
        dpath = self._get_tempdir()
        self.assertRaises(ValueError, util.get_module, "missing", dpath)

    def test_get_os_name(self):
        self.assertIn(util.get_os_name(), (util.Os_Windows, util.Os_Linux, util.Os_Mac))

    def test_get_os_version(self):
        util.get_os_version()

    def test_get_os(self):
        self.assertEqual(util.get_os(), (util.get_os_name(),
                util.get_os_version()))

    def test_get_os_microsoft(self):
        """Make sure that OS name Microsoft gets handled correctly."""
        def uname():
            return "Microsoft", "host", "Windows", "6.1.6000", "", ""

        self._set_module_attr("platform", "uname", uname)
        self.assertEqual(util.get_os(), (util.Os_Windows, "6.1.6000"))


class CommandTest(TestCase):
    """ Test Command class. """
    def test_call(self):
        def callback(*args, **kwds):
            self.__cb_args, self.__cb_kwds = args, kwds

        for args, kwds in (((), {}), ((1, 2), {"1": 1, "2": 2})):
            cmd = util.Command(callback, args, kwds)
            cmd()
            self.assertEqual(self.__cb_args, args, )
            self.assertEqual(self.__cb_kwds, kwds)



""" Unit testing functionality. """
import unittest, os.path, sys, shutil, tempfile
import warnings

import srllib.util
from mock import Mock

# This flag makes unittest omit our test methods in failure tracebacks, like the standard test methods.
__unittest = True

class TestCase(unittest.TestCase):
    """ Extended TestCase baseclass.
    
    @note: In the setUp method, the original working directory is registered
    so it can be restored in tearDown. 
    @ivar _orig_attrs: Dictionary of object attributes to reset on teardown.
    @ivar _tempfiles: List of temporary files to delete on teardown.
    @ivar _tempdirs: List of temporary directories to delete on teardown.
    """
    def __init__(self, *args, **kwds):
        unittest.TestCase.__init__(self, *args, **kwds)

    def setUp(self):
        # from conduit import mythreading
        # mythreading.registerExceptionHandler(self._exception_handler)
        (self.__connections, self._orig_attrs, self._tempfiles,
            self._tempdirs) = [], {}, [], []
                
        # Register the original working directory so this can be restored
        # at the end of the test.
        self.__cwd = os.getcwd()

    def tearDown(self):
        if self.__cwd is not None:
            os.chdir(self.__cwd)
            
        for k, v in self._orig_attrs.items():
            obj, attr = k
            val = v
            setattr(obj, attr, val)
        for sig, slt in self.__connections:
            sig.disconnect(slt)
        for ftemp in self._tempfiles:
            if isinstance(ftemp, file):
                ftemp.close()
                fname = ftemp.name
            else:
                fname = ftemp
            try: os.remove(fname)
            except OSError: pass
        for dtemp in self._tempdirs:
            if not os.path.exists(dtemp):
                continue
            srllib.util.remove_dir(dtemp, force=True, recurse=True)

        # Reset Mock register
        Mock.mockInstances.clear()

    def assertNot(self, val, msg=None):
        self.assert_(not val, msg=msg)
        
    def assertStrEqual(self, lhs, rhs, msg=None):
        self.assertEqual(str(lhs), str(rhs), msg)

    def assertStrNotEqual(self, lhs, rhs, msg=None):
        self.assertNotEqual(str(lhs), str(rhs), msg)

    def assertSortedEqual(self, lhs, rhs, msg=None):
        lhs = list(lhs)
        rhs = list(rhs)
        lhs.sort()
        rhs.sort()
        self.assertSeqEqual(lhs, rhs, msg)

    def assertSeqEqual(self, lhs, rhs, msg=None):
        if not isinstance(lhs, tuple):
            lhs = tuple(lhs)
        if not isinstance(rhs, tuple):
            rhs = tuple(rhs)
        if not lhs == rhs:
            raise self.failureException, msg or "%r != %r" % (lhs, rhs)

    def assertIs(self, lhs, rhs, msg=None):
        if not lhs is rhs:
            raise self.failureException, msg or "%r is not %r" % (lhs, rhs)

    def assertIsNot(self, lhs, rhs, msg=None):
        if lhs is rhs:
            raise self.failureException, msg or "%r is %r" % (lhs, rhs)

    def assertIn(self, val, seq, msg=None):
        if not val in seq:
            raise self.failureException, msg or "%r not in %r" % (val, seq)

    def assertNotIn(self, val, seq, msg=None):
        if val in seq:
            raise self.failureException, msg or "%r in %r" % (val, seq)

    def assertGreaterThan(self, lhs, rhs, msg=None):
        if lhs <= rhs:
            raise self.failureException, msg or "%r <= %r" % (lhs, rhs)

    def assertLessThan(self, lhs, rhs, msg=None):
        if lhs >= rhs:
            raise self.failureException, msg or "%r >= %r" % (lhs, rhs)

    def assertGreaterOrEqual(self, lhs, rhs, msg=None):
        if lhs < rhs:
            raise self.failureException, msg or "%r < %r" % (lhs, rhs)

    def assertLessOrEqual(self, lhs, rhs, msg=None):
        if lhs > rhs:
            raise self.failureException, msg or "%r > %r" % (lhs, rhs)

    def _connect_to(self, signal, slot):
        signal.connect(slot)
        self.__connections.append((signal, slot))

    def _set_attr(self, obj, attr, val):
        """ Change an attribute of some object.
        
        The original attribute of the object is recorded, so that it can be restored after the test.
        @param obj: The object.
        @param attr: Name of attribute to change
        @param val: The intended value of the attribute
        """
        if not (obj, attr) in self._orig_attrs:
            self._orig_attrs[(obj, attr)] = getattr(obj, attr)
        setattr(obj, attr, val)

    def _set_module_attr(self, mdl, attr, val):
        """ Change the attribute of a module.
        
        The original attribute of the module is recorded, so that it can be restored after the test.
        @param mdl: Name of module or module.
        @param attr: Name of attribute to change
        @param val: The intended value of the attribute
        """
        if isinstance(mdl, basestring):
            try:
                fromList = [mdl.rsplit(".", 1)[1]]
            except IndexError:
                fromList = []
            m = __import__(mdl, None, None, fromList)
        else:
            m = mdl

        self._set_attr(m, attr, val)

    def _restore_attr(self, obj, attr):
        val = self._orig_attrs.pop((obj, attr))
        setattr(obj, attr, val)

    def _restore_module_attr(self, mdl, attr):
        if isinstance(mdl, basestring):
            try:
                fromList = [mdl.rsplit(".", 1)[1]]
            except IndexError:
                fromList = []
            m = __import__(mdl, None, None, fromList)
        else:
            m = mdl
        self._restore_attr(m, attr)
    
    def _exception_handler(self, exception):
        raise exception

    def _get_datapath(self, relpath, abs=False):
        """ For subclasses: Get path to file in data directory.

        The data directory is called "Data" and resides under the test module
        (taken to be the calling module).
        @param abs: Return absolute path?
        """
        # Get calling module
        calling_mod = sys.modules[sys._getframe(1).f_globals['__name__']]
        calling_mod_dir = os.path.dirname(calling_mod.__file__)
        dpath = os.path.join(calling_mod_dir, "Data", relpath)
        if abs:
            dpath = os.path.abspath(dpath)
        return dpath

    def _get_bindatapath(self, relPath):
        if relPath.count(os.path.sep) > 1:
            parDir, remaining = relPath.split(os.path.sep, 1)
        else:
            parDir, remaining = relPath, ""
        os_name = srllib.util.get_os_name()
        if os_name == srllib.util.Os_Windows:
            parDir += "-win"
        elif os_name == srllib.util.Os_Linux:
            parDir += "-linux"
        '''
        elif Os in Darwin:
            parDir += "-osx"
        '''
        ret = os.path.join("Data", parDir, remaining)
        return ret

    def _get_tempfname(self, *args, **kwds):
        ftemp = srllib.util.create_tempfile(*args, **kwds)
        self._tempfiles.append(ftemp)
        return ftemp

    def _get_tempfile(self, *args, **kwds):
        ftemp = srllib.util.create_tempfile(close=False, *args, **kwds)
        self._tempfiles.append(ftemp)
        return ftemp

    def _get_tempdir(self, *args, **kwds):
        """ Create temporary directory that is removed on tearDown. """
        dtemp = tempfile.mkdtemp(*args, **kwds)
        self._tempdirs.append(dtemp)
        return dtemp
    
    def _get_privattr(self, obj, name, classname=None):
        """ Get a private (name mangled) attribute of an object.
        @param classname: Optionally specify the class in question where the
        private attribute is set. Otherwise it is deduced from the object's
        class.
        """
        mangled_name = self.__mangle_priv(obj, classname, name)
        return getattr(obj, mangled_name)
    
    def _set_privattr(self, obj, name, val, assert_exists=True, classname=None):
        """ Set a private (name mangled) attribute of an object.
        @param assert_exists: Assert that the attribute already exists?
        @param classname: Optionally specify the class in question where the
        private attribute is set. Otherwise it is deduced from the object's
        class.
        """
        mangled_name = self.__mangle_priv(obj, classname, name)
        if assert_exists:
            getattr(obj, mangled_name)
        setattr(obj, mangled_name, val)

    def __mangle_priv(self, obj, classname, name):
        if classname is None:
            classname = obj.__class__.__name__
        if not classname.startswith("_"):
            classname = "_" + classname
        if not name.startswith("__"):
            name = "__" + name
        else:
            warnings.warn("specifying private prefix is deprecated",
                    DeprecationWarning, stacklevel=2)

        return "%s%s" % (classname, name)

class TestPersistent(TestCase):
    pass

import stat

def _sig(st):
    return (stat.S_IFMT(st.st_mode), st.st_size, stat.S_IMODE(st.st_mode), st.st_uid, st.st_gid)

def compare_dirs(dir0, dir1, shallow=True, ignore=[]):
    """ Check that the contents of two directories match.

    Contents that mismatch and content that can't be found in one directory or can't be checked somehow are returned
    separately.
    @param shallow: Just check the stat signature instead of reading content
    @param ignore: Names of files(/directories) to ignore
    @return: Pair of mismatched and failed pathnames, respectively
    """
    mismatch = []
    error = []
    if len(os.listdir(dir0)) == 0:
        return mismatch, os.listdir(dir1)
    assert os.path.exists(dir0), "%s does not exist" % dir0
    assert os.path.exists(dir1), "%s does not exist" % dir1
    
    if dir0[-1] != os.path.sep:
        dir0 = dir0 + os.path.sep
    for dpath, dnames, fnames in os.walk(dir0):
        for ign in ignore:
            while ign in dnames:
                dnames.remove(ign)
            while ign in fnames:
                fnames.remove(ign)

        relDir = dpath[len(dir0):]
        assert relDir != dpath, dpath
        
        contents0 = dnames + fnames
        contents1 = [e for e in os.listdir(os.path.join(dir1, relDir)) if not e in ignore]

        lnth0, lnth1 = len(contents0), len(contents1)
        if lnth0 < lnth1:
            for name in contents1:
                if name not in contents0:
                    error.append(os.path.join(relDir, name))
        
        import stat
        for name in contents0:
            relPath = os.path.join(relDir, name)
            try:
                path0, path1 = os.path.join(dir0, relPath), os.path.join(dir1, relPath)
                st0, st1 = os.lstat(path0), os.lstat(path1)
                mode0, mode1 = stat.S_IMODE(st0.st_mode), stat.S_IMODE(st1.st_mode)
                mismatched = False
                s0, s1 = _sig(st0), _sig(st1)
                if name in fnames:
                    if s0 != s1:
                        sys.stderr.write("%s mismatched because %r != %r\n" % (path1, s0, s1))
                        mismatched = True
                        shutil.copytree(dir0, "/tmp/mismatch")
                    elif not shallow:
                        chksum0, chksum1 = (srllib.util.get_checksum(path0),
                                srllib.util.get_checksum(path1))
                        mismatched = srllib.util.get_checksum(path0) != \
                                srllib.util.get_checksum(path1)
                        if mismatched:
                            sys.stderr.write("%s mismatched against %s because %s != %s\n" % (path0, path1, chksum0, chksum1))
                else:
                    assert name in dnames
                    mismatched = s0[2:] != s1[2:]   # Ignore format and size
                    if mismatched:
                        # No need to traverse this directory
                        sys.stderr.write("Mismatched: %r, %r\n" % (s0[1:], s1[1:]))
                        dnames.remove(name)
                if mismatched:
                    mismatch.append(relPath)
            except OSError:
                if name in dnames:
                    dnames.remove(name)
                sys.stderr.write("Error: %s\n" % relPath)
                error.append(relPath)

    return mismatch, error

def _get_tests(specified):
    specificTests = [t for t in specified.split(",") if t]
    from glob import glob
    if not specified:
        return [os.path.splitext(os.path.basename(path))[0] for path in glob("test*.py")]

    tests = []
    for t in specificTests:
        # Prepend "test" to specified tests if necessary
        if not t.startswith("test"):
            t = "test" + t
        tests.append(t)
    return tests

'''
def _analyzePkg(pkg, rslts, ignore):
    """ Find all sub-modules of a package and analyze them. """
    import imp
    pdir = os.path.dirname(pkg.__file__)
    pkgName = pkg.__name__
    if "." in pkgName:
        # A sub package
        pkgName = pkg.__name__.split(".", 1)[1]
        topPkg = False
    else:
        topPkg = True

    for e in os.listdir(pdir):
        abspath = os.path.join(pdir, e)
        if os.path.splitext(e)[1] == ".py" or os.path.isdir(abspath) and \
                os.path.isfile(os.path.join(abspath, "__init__.py")):
            if os.path.isfile(abspath):
                if e == "__init__.py":
                    continue
                name = os.path.splitext(e)[0]
            else:
                name = e
            if not topPkg:
                qname = "%s.%s" % (pkgName, name)
            else:
                qname = name
            if qname in ignore:
                continue

            f, p, d = imp.find_module(name, pkg.__path__)
            try:
                m = imp.load_module("%s.%s" % (pkg.__name__, name), f, p, d)
            finally:
                if f is not None:
                    f.close()
            f, s, i, n, h  = coverage.analysis2(m)
            rslts[m.__name__] = (s, n, h)

            if os.path.isdir(abspath):
                # Recurse over sub-package
                _analyzePkg(m, rslts, ignore)

def _analyzeCoverage(package_name, tests, infer, ignore):
    """ Analyze code coverage by tests. """
    rslts = {}
    if not infer:
        pkg = __import__(package_name)
        _analyzePkg(pkg, rslts, ignore)
    else:
        for t in tests:
            # Get only the name of the test module, the specification may be referring to
            # a module attribute
            t = t.split(".", 1)[0]
            assert t.startswith("test")

            # See if the test specifies which modules it involves
            testMod = __import__(t)
            modules = []
            if hasattr(testMod, "_Test_Modules"):
                modules = getattr(testMod, "_Test_Modules")
            else:
                fullMname = t[4:]
                modules.append(t[4:])
                    
            for fullMname in modules:
                if fullMname in ignore:
                    continue

                if "." in fullMname:
                    mname = fullMname.rsplit(".", 1)[1]
                else:
                    mname = fullMname
                m = __import__("%s.%s" % (package_name, fullMname), globals(), locals(), [mname])
                f, s, i, n, h = coverage.analysis2(m)
                rslts[fullMname] = (s, n, h)

    totalStatements = totalNe = 0
    mnames = rslts.keys()
    mnames.sort()
    for mname in mnames:
        statements, notExecuted, formatted = rslts[mname]
        cov = 1.0 - len(notExecuted) / float(len(statements))
        print "%s:\n%s\nNo. statements: %d\nNot executed: %s\nCoverage: %f\n" % \
                (mname, "-" * (len(mname) + 1), len(statements), formatted, cov)
        totalStatements += len(statements)
        totalNe += len(notExecuted)
    print "Total coverage: %f" % (1.0 - totalNe / float(totalStatements),)
'''

def _error(msg):
    sys.stderr.write("%s\n" % msg)
    sys.exit(1)

def run_tests(package_name, gui=False, test_module=None):
    """ Use L{nose} for discovering/running tests.

    After finishing, sys.exit is called with the success status.
    @param package_name: Name of package that is tested. Used for analyzing
    coverage by tests.
    @param gui: Do the tests involve GUI code?
    @param test_module: Optionally module containing tests.
    """
    import nose

    if gui:
        import srllib.testing.qtgui, srllib.qtgui
        srllib.testing.qtgui.GuiTestCase.QApplicationClass = srllib.qtgui.Application
    sys.argv.append("--cover-package=%s" % (package_name,))
    # Can't find a better way to make nose ignore this function :(
    sys.argv.append("--exclude=%s" % ("run_tests",))

    if test_module is not None:
        try: __import__(test_module)
        except ImportError:
            _error("Couldn't import test_module ('%s')" % test_module)
        mod_loc = os.path.dirname(sys.modules[test_module].__file__)
    else:
        mod_loc = "."

    # In order to get coverage analysis of all srllib modules, "unload" them before
    # testing
    for mname in sys.modules.keys()[:]:
        if mname.split(".")[0] == "srllib":
            del sys.modules[mname]

    try:
        r = nose.run(defaultTest=mod_loc)
    finally:
        '''
        if gui:
            from PyQt4 import QtGui
            QtGui.qApp.quit()
            QtGui.qApp.processEvents()
            '''
        pass

    sys.exit(r)

from unittest import main



""" Test the signal module. """
from srllib.signal import *

from _common import *

class _Slot:
    def __call__(self):
        pass

class SignalTest(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.__emitted = False

    def testSignal(self):
        sig = self.__getSignal()
        sig("arg1", "arg2", kwd1=None)
        self.assertEqual(self.__emitted, (("arg1", "arg2"), {"kwd1": None}))

        def slot():
            self.__emitted = True

        # Just verify that function slots work
        self.__emitted = False
        sig = self.__getSignal(slot=slot)
        sig()
        self.assert_(self.__emitted)

    def testEnabling(self):
        """ Test enabling/disabling signal. """
        sig = self.__getSignal()
        self.assert_(sig.is_enabled)
        sig.disable()
        self.assertNot(sig.is_enabled)
        sig()
        self.assertNot(self.__emitted)
        sig.enable()
        self.assert_(sig.is_enabled)
        sig()
        self.assert_(self.__emitted)

    def testDisconnect(self):
        sig = self.__getSignal()
        sig.disconnect(self.__slot)
        sig()
        self.assertNot(self.__emitted)
        self.assertRaises(ValueError, sig.disconnect, self.__slot)

    def testDisconnectAllSignals(self):
        """ Test disconnecting from all signals. """
        sig0, sig1 = self.__getSignal(), self.__getSignal()
        Signal.disconnect_all_signals(self)
        sig0()
        sig1()
        self.assertNot(self.__emitted)
        # This shouldn't be an error
        Signal.disconnect_all_signals(self)
        
    def test_is_connected_yes(self):
        def slot():
            pass
        sig = self.__getSignal(slot)
        self.assert_(sig.is_connected(slot))
        
    def test_is_connected_no(self):
        def slot():
            pass
        sig = self.__getSignal()
        self.assertNot(sig.is_connected(slot))

    ''' Can't seem to make this work (force gc)
    def testDeadSlot(self):
        class Slot:
            def __call__(self, *args):
                pass
        slot = Slot()
        sig = self.__getSignal(slot=slot)
        import gc
        del slot
        gc.collect()
        sig()
    '''

    def __slot(self, *args, **kwds):
        self.__emitted = (args, kwds)

    def __getSignal(self, slot=None):
        if slot is None:
            slot = self.__slot
        sig = Signal()
        sig.connect(slot)
        return sig



""" Collection of widget classes.
"""
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

import srllib.qtgui

class _LineEditUndo(QtGui.QUndoCommand):
    __super = QtGui.QUndoCommand

    def __init__(self, edit, prev_text, prev_pos, cur_text, cur_pos, cmd_text,
            id_):
        self.__super.__init__(self, cmd_text)
        self.__edit = edit
        self.__prev = prev_text
        self.__prev_pos = prev_pos
        self.__cur = cur_text
        self.__cur_pos = cur_pos
        self.__id = id_

    def id(self):
        return self.__id

    def mergeWith(self, other):
        if not isinstance(other, _LineEditUndo) or other.id() != self.id():
            return False

        self.__cur = other.__cur
        return True

    def undo(self):
        self.__edit.setText(self.__prev)
        self.__edit.setCursorPosition(self.__prev_pos)

    def redo(self):
        self.__edit.setText(self.__cur)
        self.__edit.setCursorPosition(self.__cur_pos)

class _LineEditHelper(object):
    def __init__(self, undo_stack, undo_text, qbase):
        self.__qbase = qbase
        # Always cache state as it is before the last change, for undo
        self.__cur_text = self.text()
        self.__cur_pos = self.cursorPosition()
        srllib.qtgui.connect(self, "textEdited(const QString&)", self.__edited)
        srllib.qtgui.connect(self, "editingFinished()", self.__editing_finished)
        self.__undo_stack = undo_stack
        if undo_text is None:
            undo_text = "edit text"
        self.__undo_txt = undo_text
        self.__cur_editing = None

    def setText(self, text, undoable=False):
        if undoable:
            self.__edited(text)
        self.__qbase.setText(self, text)
        self.__cur_text = text
        self.__cur_pos = self.cursorPosition()

    def __edited(self, text):
        """ React to text being changed. """
        undo_stack = self.__undo_stack
        if undo_stack is None:
            return

        if self.__cur_editing != undo_stack.index()-1:
            # We're starting a new operation
            id_ = self.__cur_editing = undo_stack.index()
        else:
            id_ = self.__cur_editing

        # Make sure to make a copy of the text
        my_text = QtCore.QString(text)
        pos = self.cursorPosition()
        undo_stack.push(_LineEditUndo(self, self.__cur_text, self.__cur_pos, my_text,
            pos, self.__undo_txt, id_))
        self.__cur_text = my_text
        self.__cur_pos = pos

    def __editing_finished(self):
        """ We've either lost focus or the user has pressed Enter.
        """
        undo_stack = self.__undo_stack
        if undo_stack is None:
            return

        self.__cur_editing = None

class LineEdit(_LineEditHelper, QtGui.QLineEdit):
    """ Extension of QLineEdit.

    This class supports the Qt undo framework. An undo operation spans the
    interval from the user starts entering text until the widget either loses
    focus or Enter is pressed, this is considered a suitable granularity for
    this kind of widget.
    """
    __super = QtGui.QLineEdit

    def __init__(self, contents=QtCore.QString(), parent=None, undo_stack=None,
        undo_text=None):
        """ Constructor.

        @param undo_stack: Optionally specify an undo stack to manipulate as
        the line edit's text is edited.
        @param undo_text: Optionally specify descriptive text for the undo/redo
        operation.
        """
        self.__super.__init__(self, contents, parent)
        _LineEditHelper.__init__(self, undo_stack, undo_text, self.__super)

class _NumericalLineEditHelper(object):
    def __init__(self, floating_point, minimum, maximum):
        if floating_point:
            vtor = QtGui.QDoubleValidator(self)
            self.setValidator(vtor)
            if minimum is not None:
                vtor.setBottom(minimum)
            if maximum is not None:
                vtor.setTop(maximum)
        else:
            vtor = QtGui.QIntValidator(self)
            self.setValidator(vtor)
            if minimum is not None:
                if not isinstance(minimum, int):
                    raise ValueError(minimum)
                vtor.setBottom(minimum)
            if maximum is not None:
                if not isinstance(maximum, int):
                    raise ValueError(maximum)
                vtor.setTop(maximum)

class NumericalLineEdit(_NumericalLineEditHelper, LineEdit):
    """ Line edit specialization that only accepts numeric input. """
    _qbase = LineEdit

    def __init__(self, floating_point=True, contents=QtCore.QString(),
            minimum=None, maximum=None, parent=None, undo_stack=None,
            undo_text=None):
        """ Constructor.
        @param floating_point: Accept floating point numbers, not just integers?
        @param minimum: Optionally specify minimum acceptable value.
        @param maximum: Optionally specify maximum acceptable value.
        """
        self._qbase.__init__(self, contents, parent, undo_stack, undo_text)
        _NumericalLineEditHelper.__init__(self, floating_point, minimum,
            maximum)

class _CheckBoxUndo(QtGui.QUndoCommand):
    __super = QtGui.QUndoCommand

    def __init__(self, checkbox, prev_state, new_state, cmd_text):
        self.__super.__init__(self, cmd_text)
        self.__checkbox = checkbox
        self.__prev = prev_state
        self.__new = new_state

    def undo(self):
        self.__checkbox.setCheckState(self.__prev)

    def redo(self):
        self.__checkbox.setCheckState(self.__new)

class _CheckBoxHelper(object):
    def __init__(self, undo_stack, undo_text):
        srllib.qtgui.connect(self, "stateChanged(int)", self.__state_changed)
        self.__undo_stack = undo_stack
        self.__cur_state = self.checkState()
        self.__undo_txt = undo_text
        # setCheckState will emit stateChanged, so we've got to know when to
        # ignore it
        self.__setting_state = False

    def setCheckState(self, state):
        self.__setting_state = True
        try:
            self._qbase.setCheckState(self, state)
            self.__cur_state = state
        finally: self.__setting_state = False

    def __state_changed(self, state):
        if self.__setting_state:
            return
        undo_stack = self.__undo_stack
        if undo_stack is None:
            return

        # The state parameter is an integer, but a due to an inconsistency in
        # PyQt4 setCheckState will only accept Qt.CheckState objects
        if state == Qt.Checked:
            state = Qt.Checked
        elif state == Qt.Unchecked:
            state = Qt.Unchecked
        undo_stack.push(_CheckBoxUndo(self, self.__cur_state, state,
            self.__undo_txt))
        self.__cur_state = state

class CheckBox(QtGui.QCheckBox, _CheckBoxHelper):
    _qbase = __super = QtGui.QCheckBox

    def __init__(self, label=QtCore.QString(), parent=None, undo_stack=None,
        undo_text=None):
        self.__super.__init__(self, label, parent)
        _CheckBoxHelper.__init__(self, undo_stack, undo_text)



#!/usr/bin/env python
from __future__ import absolute_import

import sys, os.path

if __name__ == '__main__':
    dpath = os.path.dirname(__file__)
    if dpath:
        os.chdir(dpath)

    sys.path.insert(0, os.path.realpath(os.path.pardir))
    import srllib.testing
    srllib.testing.run_tests("srllib")



# Part of Srl Python Components
# Copyright (c) 2007 Srl Research Laboratory

# Permission is hereby granted, free of charge, to any person obtaining a copy of this software
# and associated documentation files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all copies or
# substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
""" Unit testing functionality for Qt-based components. """
from PyQt4.QtGui import *
from PyQt4.QtCore import *

from srllib.testing import *
from srllib.testing.qtgui.mock import QMock
import srllib.qtgui
from srllib import util

class WidgetController(object):
    def __init__(self, app):
        self.__app = app

    def mouseClick(self, widget):
        self.__mouseEvent(widget, QEvent.MouseButtonPress)
        self.__mouseEvent(widget, QEvent.MouseButtonRelease)

    def __mouseEvent(self, w, tp, btn=Qt.LeftButton, pos=None, kbdMods=Qt.NoModifier):
        if pos is None:
            pos = QPoint(w.width() / 2, w.height() / 2)
        e = QMouseEvent(tp, pos, btn, btn, kbdMods)
        self.__app.postEvent(w, e)
        self.__app.processEvents()

class QtTestCase(TestCase):
    """ Baseclass for tests involving Qt, but not necessarily GUI stuff.

    This class replaces QObject.connect and srllib.qtgui.connect in the setup
    phase, with an alternative implementation. The alternative implementation
    detects whether the sender is a QMock instance; if so, it uses
    QMock.connect, otherwise QObject.connect is employed. Using QMock.connect
    means that we gain control of the signal connection/emission process.
    """
    __real_connect = QObject.connect

    def setUp(self):
        TestCase.setUp(self)
        self._set_attr(QObject, "connect", self.__connect)
        self._set_attr(srllib.qtgui, "connect", self.__qtgui_connect)

    def tearDown(self):
        TestCase.tearDown(self)
        QMock.mock_clear_connections()

    def assertConnected(self, slot, emitter, signal):
        """ Assert that a slot is connected to a signal. """
        self.assert_(emitter.mock_is_connected(slot, signal), msg=
            "Slot %r not connected to signal %s of %r" % (slot, signal,
                emitter))

    @classmethod
    def __qtgui_connect(cls, sender, signal, slot):
        """ Simulate srllib.qtgui.connect.

        This function differs from standard QObject.connect by implicitly
        transforming the signal argument with SIGNAL.
        """
        cls.__connect(sender, SIGNAL(signal), slot)

    @classmethod
    def __connect(cls, sender, signal, slot):
        if isinstance(sender, QMock):
            QMock.connect(sender, signal, slot)
        else:
            cls.__real_connect(sender, signal, slot)

class GuiTestCase(QtTestCase):
    QApplicationClass = QApplication
    q_app = None

    def __init__(self, *args, **kwds):
        QtTestCase.__init__(self, *args, **kwds)

    @classmethod
    def create_application(cls):
        import sys
        cls.q_app = cls.QApplicationClass(sys.argv)
        return cls.q_app

    @classmethod
    def close_application(cls):
        if not cls.q_app.has_quit():
            # Perhaps already closed due to an exception?
            cls.q_app.quit()
            cls.q_app.processEvents()
        cls.q_app = None

    def setUp(self, widgetClass=None, *args):
        """ @param widgetClass: Optionally, pass the class of a widget to
        instantiate.
        @param args: Arguments to widget initializer.
        """
        QtTestCase.setUp(self)

        if (self.__class__.q_app is not None):
            # Reuse existing QApplication
            self._app = self.__class__.q_app
            self.__created_app = False
        else:
            # Set up QApplication for this test
            self._app = self.__class__.create_application()
            self.__created_app = True

        assert type(self._app) is self.__class__.QApplicationClass
        self._app.processEvents()
        if widgetClass is not None:
            self._widget = widgetClass(*args)
        self.__qtConns = []
        self._widgetController = WidgetController(self._app)

    def tearDown(self):
        QtTestCase.tearDown(self)

        self._widget.close()
        for sender, sig, slt in self.__qtConns:
            QObject.disconnect(sender, sig, slt)
        if self.__created_app:
            self.close_application()

    def _connectToQt(self, sender, signal, slot):
        QObject.connect(sender, SIGNAL(signal), slot)
        self.__qtConns.append((sender, SIGNAL(signal), slot))

    def _scheduleCall(self, func, interval=0):
        """ Schedule function with Qt event loop to be called after a certain interval.

        @param func: Function to execute.
        @param interval: Interval in seconds.
        """
        QTimer.singleShot(int(interval * 1000), func)



""" Various utility functionality.
@var Os_Linux: Identifier for the Linux operating system.
@var Os_Windows: Identifier the Windows operating system.
@var Os_Mac: Identifier for the Mac OS operating system.
OsCollection_Posix: Collection of identifiers for POSIX OSes.
"""
import stat, shutil, os.path, imp, platform, fnmatch, sys, errno, \
        codecs
try: import hashlib
except ImportError: import sha
import functools

from srllib.error import *

class PermissionsError(SrlError):
    """ Filesystem permissions error.
    @ivar reason: Original reason for error.
    """
    def __init__(self, path, reason=None):
        SrlError.__init__(self, "Unsufficient permissions for %s" % (path,))
        self.reason = reason

def no_op(*args, **kwds):
    """ Utility no-op function that accepts any arguments/keywords. """
    pass

Checksum_Hex, Checksum_Binary = 0, 1

def get_checksum(path, format=Checksum_Hex, callback=no_op):
    """ Obtain the sha1 checksum of a file or directory.

    If path points to a directory, a collective checksum is calculated
    recursively for all files in the directory tree.
    @param path: Path to file or directory
    @param format: One of L{Checksum_Hex}, L{Checksum_Binary}.
    @param callback: Optionally supply a callback to be periodically called. Raise
    Canceled from this to cancel the operation.
    @return: If hexadecimal, a 40 byte hexadecimal digest. If binary, a 20byte
    binary digest.
    @raise ValueError: Invalid format.
    @raise Canceled: The callback indicated that the operation should be
    canceled.
    """
    if format not in (Checksum_Hex, Checksum_Binary):
        raise ValueError("Invalid format")

    def performSha1(path, shaObj):
        f = file(path)
        try:
            while True:
                callback()
                bytes = f.read(8192)
                shaObj.update(bytes)
                if len(bytes) < 8192:
                    break
        finally:
            f.close()

    try: shaObj = hashlib.sha1()
    except NameError: shaObj = sha.new()
    if format == Checksum_Hex:
        shaMthd = shaObj.hexdigest
    else:
        shaMthd = shaObj.digest

    if os.path.isdir(path):
        for dpath, dnames, fnames in walkdir(path):
            for fname in fnames:
                performSha1(os.path.join(dpath, fname), shaObj)
    else:
        performSha1(path, shaObj)

    return shaMthd()

def get_module(name, path):
    """ Search for a module along a given path and load it.
    @param name: Module name.
    @param path: Path of directories to search. A single-directory path can be
    expressed as a string.
    @raise ValueError: Module not found.
    """
    if isinstance(path, basestring):
        path = [path]
    try:
        file_, fname, extra = imp.find_module(name, path)
        mod = imp.load_module(name, file_, fname, extra)
    except ImportError:
        raise ValueError(name)
    return mod

#{ Operating-system logic

Os_Linux = "linux"
Os_Windows = "windows"
Os_Mac = "darwin"
Os_Solaris = "sunos"

OsCollection_Posix = (Os_Linux, Os_Mac, Os_Solaris)
Os_Posix = OsCollection_Posix   # Backwards-compat

def get_os():
    """ Get the current operating system.

    Lower-case strings are used to identify operating systems.
    @return: A pair of OS identifier and OS release (e.g. "xp") strings.
    """
    name, host, rls, ver, mach, proc = platform.uname()
    name = name.lower()
    if name == "microsoft":
        # On some Windows versions, it comes out on a different form than usual
        name = rls.lower()
        rls = ver

    return name, rls

def get_os_name():
    """ Get the name of the current operating system.

    This convenience function simply returns the first element of the tuple
    returned by L{get_os}.
    """
    return get_os()[0]

def get_os_version():
    """ Get the version of the current operating system.

    This convenience function simply returns the second element of the tuple
    returned by L{get_os}.
    """
    return get_os()[1]

#}

#{ Filesystem utilities

def replace_root(path, new_root, orig_root=None):
    """ Replace one root directory component of a pathname with another.
    @param path: The pathname.
    @param new_root: The root to replace the old root component with.
    @param orig_root: The original root component to replace, if C{None} the
    first directory component.
    @return: The new pathname.
    @raise ValueError: Original root not found in pathname.
    """
    # Make sure to normalize path, e.g. to convert UNIX-style separators on
    # Windows
    path = os.path.normpath(path)
    # If the new root is empty, leave it be
    if new_root:
        new_root = os.path.normpath(new_root)
    if os.path.sep not in path:
        # No directory component
        return path

    if orig_root is None:
        # Find first dir component
        orig_root = ""
        assert path
        for c in path:
            if c != os.path.sep:
                orig_root += c
            else:
                break
    else:
        orig_root = os.path.normpath(orig_root)
    if not orig_root or orig_root[-1] != os.path.sep:
        orig_root += os.path.sep
    if not path.startswith(orig_root):
        raise ValueError("Path '%s' doesn't start with specified original root '%s'" %
                (path, orig_root))

    relPath = path[len(orig_root):]
    return os.path.join(new_root, relPath)

def _raise_permissions(func):
    """ Decorator for raising PermissionsError upon filesystem access error. """
    @functools.wraps(func)
    def wrapper(*args, **kwds):
        try: return func(*args, **kwds)
        except EnvironmentError, err:
            if err.errno == errno.EACCES:
                import stat
                mode = stat.S_IMODE((os.lstat(os.path.dirname(err.filename)).st_mode)) & stat.S_IEXEC
                raise PermissionsError(err.filename)
            else:
                raise

    return wrapper

class DirNotEmpty(SrlError):
    pass

@_raise_permissions
def remove_dir(path, ignore_errors=False, force=False, recurse=True):
    """ Remove directory, optionally a whole directory tree (recursively).

    @param ignore_errors: Ignore failed deletions?
    @param force: On Windows, force deletion of read-only files?
    @param recurse: Delete also contents, recursively?
    @raise ValueError: The directory doesn't exist.
    @raise PermissionsError: Missing file-permissions.
    @raise DirNotEmpty: Directory was not empty, and recurse was not specified.
    """
    def rmdir(path):
        if force:
            # When in force mode, make the directory writeable if necessary
            mode = get_file_permissions(path)
            if not mode & stat.S_IWRITE:
                mode |= stat.S_IWRITE
                chmod(path, mode)

        assert not os.listdir(path), os.listdir(path)
        try: os.rmdir(path)
        except OSError:
            if not ignore_errors:
                raise

    def handle_err(dpath):
        if not force:
            raise PermissionsError(dpath)
        os.chmod(dpath, stat.S_IEXEC | stat.S_IREAD)
        return True

    if not os.path.exists(path):
        raise ValueError("Directory doesn't exist: %s" % path)

    if recurse:
        for dpath, dnames, fnames in walkdir(path, topdown=False, errorfunc=
                handle_err):
            if force:
                # Make the directory writeable if necessary -- on Unix you
                # can't delete children of read-only directories
                mode = get_file_permissions(dpath)
                if not mode & stat.S_IWRITE:
                    mode |= stat.S_IWRITE
                    chmod(dpath, mode)
            
            for d in dnames:
                abspath = os.path.join(dpath, d)
                if not os.path.islink(abspath):
                    rmdir(abspath)
                else:
                    # Better to remove symlinks as files, since it doesn't
                    # matter if they're broken
                    remove_file(abspath, force=force)
            for f in fnames:
                remove_file(os.path.join(dpath, f), force=force)
    else:
        if os.listdir(path):
            raise DirNotEmpty
    rmdir(path)

@_raise_permissions
def get_file_permissions(path):
    """ Get permissions flags (bitwise) for a file/directory.

    Links are not dereferenced.
    """
    return stat.S_IMODE(os.lstat(path).st_mode)

@_raise_permissions
def move_file(src, dest, force=False):
    """ Move a file, cross-platform safe.

    This a simple convenience wrapper, for handling snags that may arise on
    different platforms.
    @param force: On Windows, overwrite write-protected destination?
    """
    if get_os_name() == Os_Windows and os.path.isfile(dest):    #pragma: optional
        # Necessary on Windows
        if force:
            dstMode = get_file_permissions(dest)
            if not dstMode & stat.S_IWRITE:
                chmod(dest, dstMode | stat.S_IWRITE)
        os.remove(dest)
    shutil.move(src, dest)

def clean_path(path):
    """ Return a clean, absolute path. """
    return os.path.abspath(os.path.normpath(path))

def _walkdir_handle_err(path):
    raise PermissionsError(path)

@_raise_permissions
def walkdir(path, errorfunc=None, topdown=True, ignore=None):
    """ Directory tree generator.

    This function works in much the same way as os.walk, but expands somewhat
    on that.
    @param path: Directory to traverse.
    @param errorfunc: Function to handle error when a directory can't be
    traversed. Return False from this to ignore directory.
    @param topdown: Traverse in topdown fashion?
    @param ignore: Optionally provide a set of filenames to ignore.
    @raise PermissionsError: Missing permission to traverse directory.
    """
    if errorfunc is None:
        errorfunc = _walkdir_handle_err

    mode = get_file_permissions(path)
    if not mode & stat.S_IREAD:
        if not errorfunc(path):
            # Ignore directory
            return

    if ignore is None:
        ignore = []

    dnames, fnames = [], []
    for e in os.listdir(path):
        if e in ignore:
            continue

        if os.path.isdir(os.path.join(path, e)):
            dnames.append(e)
        else:
            fnames.append(e)

    if topdown:
        yield path, dnames, fnames
    for d in dnames:
        if os.path.islink(os.path.join(path, d)):
            continue
        for x in walkdir(os.path.join(path, d), errorfunc=errorfunc,
                topdown=topdown):
            yield x
    if not topdown:
        yield path, dnames, fnames

@_raise_permissions
def _copy_file(srcpath, dstpath, callback, fs_mode=None):
    if not os.path.exists(srcpath):
        raise MissingSource(srcpath)

    st = os.stat(srcpath)
    sz = float(st.st_size)
    if sz == 0:
        # Just create the destination file
        file(dstpath, "wb").close()
        callback(100)
    else:
        src = file(srcpath, "rb")
        dst = file(dstpath, "wb")
        try:
            while True:
                bytes = src.read(8192)
                dst.write(bytes)
                bytesRead = len(bytes)
                callback(bytesRead / sz * 100)
                if bytesRead < 8192:
                    break
        finally:
            src.close()
            dst.close()

    if fs_mode is None:
        shutil.copystat(srcpath, dstpath)
    else:
        chmod(dstpath, fs_mode)

def copy_file(sourcepath, destpath, callback=no_op, fs_mode=None):
    """ Copy a file.
    @param sourcepath: Source file path.
    @param destpath: Destination file path.
    @param callback: Optional callback to be invoked periodically with progress
    status.
    @param fs_mode: Optionally, specify mode to create destination file with.
    @raise MissingSource: Source file is missing.
    raise PermissionsError: Missing filesystem permissions.
    """
    _copy_file(sourcepath, destpath, callback, fs_mode=fs_mode)

@_raise_permissions
def remove_file(path, force=False):
    """ Remove a file.
    @param force: Force deletion of read-only file?
    @raise PermissionsError: Missing file-permissions.
    """
    if force:
        # Make the necessary file/directory writeable if it isn't already
        old_mode = get_file_permissions(path)
        if not old_mode & stat.S_IWRITE:
            chmod(path, old_mode | stat.S_IWRITE)

    os.remove(path)

def remove_file_or_dir(path, force=False, recurse=True):
    """ Remove a filesystem object, whether it is a file or a directory.
    @param force: On Windows, force deletion of read-only files?
    @param recurse: Delete recursively, if a directory?
    @raise PermissionsError: Missing file-permissions.
    @raise DirNotEmpty: Directory was not empty, and recurse was not specified.
    """
    if os.path.isdir(path):
        remove_dir(path, force=force, recurse=recurse)
    else:
        remove_file(path, force=force)

class DestinationExists(SrlError):
    pass

class MissingSource(SrlError):
    pass

CopyDir_New, CopyDir_Delete, CopyDir_Merge = range(3)

class _CopyDirCallback:
    """ Translate from per-file progress to directory progress. """
    def __init__(self, total, callback):
        self.__total, self.__callback = (float(total), callback)
        self.__progress = 0.0

    def __call__(self, progress):
        self.__callback(self.__progress + progress*self.__cur_mult)

    def start_file(self, size):
        if self.__total != 0:
            self.__cur_mult = size / self.__total
        else:
            self.__cur_mult = 0

    def end_file(self):
        self.__progress += 100 * self.__cur_mult

@_raise_permissions
def copy_dir(sourcedir, destdir, callback=no_op, ignore=[], mode=CopyDir_New,
        copyfile=None, fs_mode=None):
    """ Copy a directory and its contents.

    Custom File Copying
    ===================
    It is possible to specialize file copying by passing a function for the
    I{copyfile} parameter. This function receives for its two first arguments
    pathnames for the source and destination file respectively, and then a
    callback function. This function is not called with directories or symlinks.

    The callback should be called periodically during copying, with progress
    percentage as a float.
    @param sourcedir: Source directory.
    @param destdir: Destination directory.
    @param callback: Optional callback to be invoked periodically with progress
    status. Raise L{Canceled} from this to cancel.
    @param ignore: Optional list of filename glob patterns to ignore.
    @param mode: Specify the copying mode. CopyDir_New means to refuse copying
    onto an existing directory, CopyDir_Delete means delete existing
    destination, CopyDir_Merge means copying contents of source directory into
    destination directory.
    @param copyfile: Optionally supply a custom function for copying individual
    files.
    @param fs_mode: The numeric mode to create files/directories with.
    @raise MissingSource: The source directory doesn't exist.
    @raise DirectoryExists: The destination directory already exists (and
    mode is CopyDir_New).
    @raise PermissionsError: Missing permission to perform operation.
    @raise Canceled: The callback requested canceling.
    """
    if os.path.exists(destdir):
        if mode == CopyDir_New:
            raise DestinationExists(destdir)
        elif mode == CopyDir_Delete:
            remove_file_or_dir(destdir)

    def filter(names):
        """ Filter list of filesystem names in-place. When using os.walk,
        directories removed from the list won't be traversed. """
        for ptrn in ignore:
            for name in names[:]:
                if fnmatch.fnmatch(name, ptrn):
                    names.remove(name)
        return names

    def update_mode(src, dst):
        """Potentially copy mode from source to destination."""
        if get_os_name() == Os_Windows:
            # Won't work on Windows
            return

        if fs_mode is None:
            # Only copy if mode isn't already specified
            shutil.copystat(src, dst)

    if not os.path.exists(sourcedir):
        raise MissingSource(sourcedir)

    mkdir_kwds = {}

    if not os.path.exists(destdir):
        if fs_mode is not None:
            mkdir_kwds["mode"] = fs_mode
        os.makedirs(destdir, **mkdir_kwds)
    update_mode(sourcedir, destdir)

    # We figure out the total number of bytes, for computing progress
    allbytes = 0
    for dpath, dnames, fnames in walkdir(sourcedir):
        for d in filter(dnames):
            allbytes += 1
        for f in filter(fnames):
            allbytes += os.lstat(os.path.join(dpath, f)).st_size

    if copyfile is None:
        copyfile = _copy_file
    mycallback = _CopyDirCallback(allbytes, callback)

    # First invoke the callback with a progress of 0
    callback(0)
    for dpath, dnames, fnames in walkdir(sourcedir):
        for d in filter(dnames):
            srcpath = os.path.join(dpath, d)
            dstpath = replace_root(srcpath, destdir, sourcedir)
            if os.path.exists(dstpath):
                remove_file_or_dir(dstpath)
            os.mkdir(dstpath, **mkdir_kwds)
            update_mode(srcpath, dstpath)
            mycallback.start_file(1)
            mycallback(100)
            mycallback.end_file()
        for f in filter(fnames):
            srcpath = os.path.join(dpath, f)
            dstpath = replace_root(srcpath, destdir, sourcedir)
            if os.path.exists(dstpath):
                remove_file_or_dir(dstpath)
            if hasattr(os, "symlink") and os.path.islink(srcpath):
                linktgt = os.readlink(srcpath)
                os.symlink(linktgt, dstpath)
                continue

            mycallback.start_file(os.lstat(srcpath).st_size)
            copyfile(srcpath, dstpath, mycallback, fs_mode=fs_mode)
            mycallback.end_file()

def create_tempfile(suffix="", prefix="tmp", close=True, content=None,
        encoding=None, dir=None):
    """ Create temporary file.

    The file is opened in R/W mode.
    @param suffix: Optional filename suffix.
    @param prefix: Optional filename prefix.
    @param close: Close the file after creating it?
    @param content: Optional content to write to file.
    @param encoding: Specify text encoding for file.
    @param dir: Optionally specify directory for the file.
    @return: If C{close} path to created temporary file, else temporary file.
    """
    import tempfile
    (fd, fname) = tempfile.mkstemp(suffix=suffix, prefix=prefix, dir=dir)
    # File should not be automatically deleted
    os.close(fd)

    if not close or content:
        # Open file directly instead of using fdopen, since the latter will
        # return a file with a bogus name
        try:
            if encoding is None:
                f = file(fname, "w+")
            else:
                f = codecs.open(fname, "w+", encoding)
            try:
                if content:
                    f.write(content)
                    # Make sure to reset the file position!
                    f.seek(0)
            except:
                f.close()
                raise
            if close:
                f.close()
                return fname
        except:
            # Make sure to delete created file
            os.remove(fname)
            raise
        return f
    return fname

def create_file(name, content="", binary=False, encoding=None, close=True):
    """ Create a file, with optional content.
    @param name: Filename.
    @param content: Optional content to write to file.
    @param binary: Create file in binary mode (makes a difference on Windows)?
    @param encoding: Specify text encoding of file content.
    @param close: Close file? If true, the file is returned, else its name is
    returned.
    @return: Path to created file.
    """
    mode = "w"
    if binary:
        mode += "b"
    if encoding is None:
        f = file(name, mode)
    else:
        f = codecs.open(name, mode, encoding)
    try:
        if content:
            f.write(content)
    except:
        f.close()
        raise
    if close:
        f.close()
        return name
    return f

def read_file(name, binary=False, encoding=None):
    """ Read content of file.
    @param name: Filename.
    @param binary: Open in binary mode (makes a difference on Windows)?
    @param encoding: Optionally specify text encoding of file content.
    @return: File content as string.
    """
    if not binary:
        mode = "rb"
    else:
        mode = "r"
    if encoding is None:
        f = file(name, mode)
    else:
        f = codecs.open(name, mode=mode, encoding=encoding)
    try: content = f.read()
    finally: f.close()

    return content

def _sig(st):
    return (stat.S_IFMT(st.st_mode), st.st_size, stat.S_IMODE(st.st_mode),
            st.st_uid, st.st_gid)

def compare_dirs(dir0, dir1, shallow=True, ignore=[], filecheck_func=None):
    """ Check that the contents of two directories match.

    Contents that mismatch and content that can't be found in one directory or
    can't be checked somehow are returned separately.
    @param shallow: Just check the stat signature instead of reading content
    @param ignore: Names of files(/directories) to ignore
    @param filecheck_func: Optionally provide function for deciding whether
    two files are alike.
    @raise ValueError: One of the directories are missing.
    @return: Pair of mismatched and failed pathnames, respectively
    """
    def checkfiles(path0, path1):
        chksum0, chksum1 = (get_checksum(path0, format=Checksum_Binary),
            get_checksum(path1, format=Checksum_Binary))
        r = chksum0 == chksum1
        if not r:
            sys.stderr.write("%s mismatched against %s because %s != %s\n" %
                    (path0, path1, chksum0, chksum1))
        return r

    if filecheck_func is None:
        filecheck_func = checkfiles
    mismatch = []
    error = []
    if not os.path.exists(dir0):
        raise ValueError("First directory missing: '%s'" % (dir0,))
    if not os.path.exists(dir1):
        raise ValueError("Second directory missing: '%s'" % (dir1,))
    if len(os.listdir(dir0)) == 0:
        # Indicate any files in dir1 as erroneus
        return mismatch, os.listdir(dir1)

    if dir0[-1] != os.path.sep:
        dir0 = dir0 + os.path.sep

    def handle_err(dpath):
        # Don't traverse this directory
        return False
    for dpath, dnames, fnames in walkdir(dir0, handle_err):

        # Filter out files to ignore
        for ign in ignore:
            while ign in dnames:
                dnames.remove(ign)
            while ign in fnames:
                fnames.remove(ign)

        reldir = dpath[len(dir0):]
        assert reldir != dpath, dpath

        contents0 = dnames + fnames
        contents1 = [e for e in os.listdir(os.path.join(dir1, reldir)) if e not
            in ignore]

        lnth0, lnth1 = len(contents0), len(contents1)
        if lnth0 < lnth1:
            for name in contents1:
                if name not in contents0:
                    error.append(os.path.join(reldir, name))

        for name in contents0:
            relpath = os.path.join(reldir, name)
            try:
                path0, path1 = os.path.join(dir0, relpath), os.path.join(dir1,
                        relpath)
                st0, st1 = os.lstat(path0), os.lstat(path1)
                mismatched = False
                s0, s1 = _sig(st0), _sig(st1)
                if name in fnames:
                    if s0 != s1:
                        sys.stderr.write("%s mismatched because %r != %r\n" %
                                (path1, s0, s1))
                        mismatched = True
                    elif not shallow:
                        mismatched = not filecheck_func(path0, path1)
                else:
                    assert name in dnames
                    mismatched = s0[2:] != s1[2:]   # Ignore format and size
                    if mismatched:
                        # No need to traverse this directory
                        sys.stderr.write("Mismatched: %r, %r\n" % (s0[1:],
                            s1[1:]))
                        dnames.remove(name)
                if mismatched:
                    mismatch.append(relpath)
            except OSError:
                if name in dnames:
                    dnames.remove(name)
                error.append(relpath)

    return mismatch, error

@_raise_permissions
def chmod(path, mode, recursive=False):
    """ Wrapper around os.chmod, which allows for recursive modification of a
    directory tree.
    @param path: Path to modify.
    @param mode: New permissions mode.
    @param recursive: Apply recursively, if directory?
    @raise PermissionsError: Missing file-permissions.
    """
    made_exec = []

    def _chmod(path, fixexec=False):
        newmode = mode
        if fixexec and os.path.isdir(path) and not mode & stat.S_IEXEC:
            # Make the directory accessible
            newmode |= stat.S_IEXEC
            made_exec.append(path)

        os.chmod(path, newmode)

    if recursive and os.path.isdir(path):
        seen = set()
        for dpath, dnames, fnames in walkdir(path):
            assert dpath not in seen, "Already entered %s" % (dpath,)
            seen.add(dpath)
            _chmod(dpath, fixexec=True)

            for d in dnames[:]:
                # Chmod empty dirs now
                os.listdir(os.path.join(dpath, d))
                if not os.listdir(os.path.join(dpath, d)):
                    _chmod(os.path.join(dpath, d))
                    dnames.remove(d)
            for f in fnames:
                _chmod(os.path.join(dpath, f))

        # Now finalize the executable bit in depth-first fashion
        for dpath in reversed(made_exec):
            _chmod(dpath)
    else:
        _chmod(path)

def resolve_path(executable):
    """Resolve name of executable into absolute path.

    On Windows, we will use win32api.FindExecutable if preferable, otherwise
    we will use the standard algorithm. Note on this platform, though, that if
    the executable doesn't have one of the possible extensions
    of an executable, as defined in $PATHEXTS, we will search the path for a
    combination of the filename and any extension in $PATHEXTS (e.g., .exe).
    @raise NotFound: The executable was not found in path.
    @raise ValueError: An invalid filename was passed.
    """
    if os.path.sep in executable:
        raise ValueError("Invalid filename: %s" % executable)
    if get_os_name() == Os_Windows:    # pragma: optional
        try: import win32api, pywintypes
        except ImportError: pass
        else:
            try: exepath = win32api.FindExecutable(executable)[1]
            except pywintypes.error: raise NotFound(executable)
            return exepath

    # Default solution, search path ourselves

    path = os.environ.get("PATH", "").split(os.pathsep)
    if get_os_name() == Os_Windows:
        # PATHEXTS tells us which extensions an executable may have
        path_exts = os.environ.get("PATHEXTS", ".exe;.bat").split(";")
        has_ext = os.path.splitext(executable)[1] in path_exts
        if not has_ext:
            exts = path_exts
        else:
            # Don't try to append any extensions
            exts = [""]
    else:
        # General, executables don't have extensions
        exts = [""]
    for d in path:
        try:
            for ext in exts:
                exepath = os.path.join(d, executable + ext)
                if os.access(exepath, os.X_OK):
                    return exepath
        except OSError:
            pass

    raise NotFound(executable)

#}

class Command(object):
    """ Curry a function with associated parameters.

    The rationale for this class is to be able to store a function and certain
    parameters to supply it with.
    """
    def __init__(self, callback, callback_args=(), callback_kwds={}):
        self.__cb = callback
        self.__cb_args, self.__cb_kwds = callback_args, callback_kwds

    def __call__(self):
        self.__cb(*self.__cb_args, **self.__cb_kwds)



import inspect

import srllib.inspect

from _common import *

class _MyClass(object):
    cvar = "cvar"
    
    def __init__(self):
        self.attr = "attr"
    
    def mthd(self):
        pass

class InspectTest(TestCase):
    def test_get_members_instance(self):
        """ Test get_members function on a class instance. """
        obj = _MyClass()
        members = srllib.inspect.get_members(obj)
        expected = {"mthd": obj.mthd, "attr": obj.attr, "cvar": obj.cvar}
        self.__verify(members, expected)
        
    def test_get_members_instance_method(self):
        """ Test getting methods of a class instance. """
        obj = _MyClass()
        members = srllib.inspect.get_members(obj, inspect.isroutine)
        expected = {"mthd": obj.mthd}
        self.__verify(members, expected)
        
    def test_get_members_class(self):
        """ Test get_members on a class. """
        members = srllib.inspect.get_members(_MyClass)
        expected = {"mthd": _MyClass.mthd, "cvar": _MyClass.cvar}        
        self.__verify(members, expected)
    
    def __verify(self, got, expected):
        for k in got.keys():
            if k.startswith("__") and k.endswith("__"):
                got.pop(k)
        self.assertEqual(got, expected,
            "Expected members %r, got %r" % (expected, got))


""" Mock classes for use with Qt.
"""
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import QtCore, QtGui
import warnings

from srllib.qtgui import widgets

from srllib.testing.qtgui.mock import Mock, QMock, QWidgetMock, QDialogMock

class QDockWidgetMock(QWidgetMock):
    _MockRealClass = QDockWidget

class QMenuMock(Mock):
    _MockRealClass = QMenu

    def __init__(self, title=None):
        Mock.__init__(self)
        self.mock_entries = []
        self.__cur_sect = []
        self.mock_entries.append(self.__cur_sect)
        self.mock_title = title

    def mock_get_actions(self):
        """ Get all action entries, indexed on name.

        To find actions, they must be instances of QAction or L{QActionMock}.
        """
        actions = {}
        for sect in self.mock_entries:
            for e in sect:
                if isinstance(e, (QAction, QActionMock)):
                    actions[e.text().replace("&", "").replace(" ", "")] = e
        return actions

    def addAction(self, action):
        self.__cur_sect.append(action)

    def addMenu(self, title):
        menu = QMenuMock()
        self.__cur_sect.append(QMenuMock(title))
        return menu

    def addSeparator(self):
        self.__cur_sect = []
        self.mock_entries.append(self.__cur_sect)

class QActionMock(QMock):
    _MockRealClass = QAction

    def __init__(self, text=None, parent=None):
        QMock.__init__(self, parent)
        self.__enabled = True
        self.__checked = False
        self.__text = text

    def text(self):
        return self.__text

    def setText(self, text):
        self.__text = text

    def setEnabled(self, enabled):
        self.__enabled = enabled

    def isEnabled(self):
        return self.__enabled

    def setChecked(self, checked):
        self.__checked = checked

    def isChecked(self):
        return self.__checked

    def trigger(self):
        self.emit(SIGNAL("triggered()"))

class QActionGroupMock(QMock):
    _MockRealClass = QActionGroup

    def __init__(self, parent=None):
        QMock.__init__(self, parent)
        self.__enabled = True
        self.__actions = []

    def isEnabled(self):
        return self.__enabled

    def setEnabled(self, enabled):
        self.__enabled = enabled

    def addAction(self, action):
        self.__actions.append(action)

class QToolButtonMock(QMock):
    _MockRealClass = QToolButton

    def __init__(self):
        QMock.__init__(self)
        self.__enabled = True

    def setEnabled(self, enabled):
        self.__enabled = enabled

    def click(self):
        self.emit(SIGNAL("clicked()"))

    def isEnabled(self):
        return self.__enabled

class QLabelMock(QWidgetMock):
    _MockRealClass = QLabel

    def __init__(self, text=""):
        QWidgetMock.__init__(self)
        self.__text = text

    def text(self):
        return self.__text

    def setText(self, text):
        self.__text = text

class _ItemMockBase(Mock):
    def __init__(self, model):
        self.model = model

class QPixmapMock(QMock):
    _MockRealClass = QPixmap

class QComboBoxMock(QWidgetMock):
    _MockRealClass = QComboBox

    def __init__(self, *args, **kwds):
        QWidgetMock.__init__(self, *args, **kwds)
        self.mock_items = []

    def addItem(self, item):
        self.mock_items.append(item)

    def addItems(self, items):
        self.mock_items.extend(items)

    def clear(self):
        self.mock_items = []

class QStatusBarMock(QWidgetMock):
    _MockRealClass = QStatusBar

    def __init__(self, *args, **kwds):
        QWidgetMock.__init__(self, *args, **kwds)
        self.mock_permanent_widgets = []

    def addPermanentWidget(self, widget):
        self.mock_permanent_widgets.append(widget)

class QLineEditMock(QMock):
    _MockRealClass = QLineEdit

    def __init__(self, *args, **kwds):
        kwds.setdefault("returnValues", {}).setdefault("text", "")
        QMock.__init__(self, *args, **kwds)
        self.mockSetReturnValue("cursorPosition", len(self.text()))

    def setText(self, text):
        self.mockSetReturnValue("text", text)
        self.emit(SIGNAL("textChanged(const QString&)"), text)
        self.setCursorPosition(len(text))

    def setCursorPosition(self, pos):
        old_pos = self.cursorPosition()
        self.mockSetReturnValue("cursorPosition", pos)
        self.emit(SIGNAL("cursorPositionChanged(int, int)"),
            old_pos, pos)

    def setReadOnly(self, readonly):
        self.mockSetReturnValue("isReadOnly", readonly)

class QToolBoxMock(QWidgetMock):
    _MockRealClass = QToolBox

class QTreeWidgetMock(QWidgetMock):
    _MockRealClass = QTreeWidget

    def __init__(self, *args, **kwds):
        QWidgetMock.__init__(self, *args, **kwds)
        self.mock_items = []

    def addTopLevelItem(self, item):
        self.mock_items.append(item)

    def clear(self):
        self.mock_items = []

class QTreeWidgetItemMock(QWidgetMock):
    _MockRealClass = QTreeWidgetItem


class QButtonGroupMock(QMock):
    _MockRealClass = QButtonGroup

    def __init__(self, *args, **kwds):
        QMock.__init__(self, *args, **kwds)
        self.__btns = {}
        self.__checked = None

    def mock_set_checked(self, btn):
        self.__checked = btn
        for b in self.__btns.values():
            if b.isChecked() and b is not btn:
                b.setChecked(False)

    def addButton(self, btn, id=None):
        if id is None:
            if self.__btns:
                id = max(self.__btns) + 1
            else:
                id = 0
        self.__btns[id] = btn
        btn.mock_group = self

    def button(self, id):
        return self.__btns[id]

    def buttons(self):
        keys = sorted(self.__btns.keys())
        return [self.__btns[k] for k in keys]

    def checkedButton(self):
        return self.__checked

    def checkedId(self):
        for i, b in self.__btns.items():
            if b is self.__checked:
                return i

class QGroupBoxMock(QMock):
    _MockRealClass = QGroupBox

    def __init__(self):
        QMock.__init__(self)
        self.__checked = True
        self.__checkable = False

    def isCheckable(self):
        return self.__checkable

    def setCheckable(self, checkable):
        self.__checkable = checkable

    def isChecked(self):
        return self.__checkable and self.__checked

class _ButtonMockBase(QWidgetMock):
    def click(self):
        self.emit(SIGNAL("clicked()"))

class QPushButtonMock(_ButtonMockBase):
    _MockRealClass = QPushButton

class QRadioButtonMock(QMock):
    _MockRealClass = QRadioButton

    def __init__(self):
        QMock.__init__(self)
        self.__checked = False
        self.mock_group = None

    def setChecked(self, checked):
        self.__checked = checked
        if self.mock_group is not None:
            self.mock_group.mock_set_checked(self)

    def isChecked(self):
        return self.__checked

class QCheckBoxMock(QMock):
    _MockRealClass = QCheckBox

    def __init__(self, *args, **kwds):
        kwds.setdefault("returnValues", {}).setdefault("checkState",
            Qt.Unchecked)
        QMock.__init__(self, *args, **kwds)
        self.__checked = False

    def setCheckState(self, checkState):
        self.mockSetReturnValue("checkState", checkState)
        self.emit(QtCore.SIGNAL("stateChanged(int)"), int(checkState))

    def setChecked(self, checked):
        warnings.warn("Deprecated", DeprecationWarning)
        self.__checked = checked

    def isChecked(self):
        warnings.warn("Deprecated", DeprecationWarning)
        return self.__checked

class QListWidgetMock(QWidgetMock):
    """ Mock QListWidget.

    @ivar mock_items: List of items.
    """
    _MockRealClass = QListWidget

    def __init__(self):
        QWidgetMock.__init__(self)
        self.mock_items = []

    def addItem(self, text):
        self.mock_items.append(text)

    def count(self):
        return len(self.mock_items)

class QTableWidgetMock(QWidgetMock):
    _MockRealClass = QTableWidget

    def __init__(self):
        QWidgetMock.__init__(self)
        self.__row_count = self.__col_count = 0
        self.mock_items = []
        self.mock_cell_widgets = []

    def rowCount(self):
        return self.__row_count

    def setRowCount(self, count):
        self.__row_count = count

    def setColumnCount(self, count):
        self.__col_count = count

    def columnCount(self):
        return self.__col_count

    def item(self, row, col):
        return self.mock_items[row].get(col)

    def setItem(self, row, col, item):
        items = self.mock_items
        while row >= len(items):
            items.append({})
        items[row][col] = item

    def takeItem(self, row, col):
        return self.mock_items[row].pop(col)

    def cellWidget(self, row, col):
        return self.mock_cell_widgets[row].get(col)

    def setCellWidget(self, row, col, widget):
        items = self.mock_cell_widgets
        while row >= len(items):
            items.append({})
        items[row][col] = widget

class QTableWidgetItemMock(QMock):
    _MockRealClass = QTableWidgetItem

class QIconMock(QMock):
    _MockRealClass = QIcon

class QFileMock(QMock):
    _MockRealClass = QFile

class QTextStreamMock(QMock):
    _MockRealClass = QTextStream

class QMessageBoxMock(QDialogMock):
    Yes, No, Cancel = range(3)


class BrowseDirectoryMock(QWidgetMock):
    def __init__(self, baseargs=(), basekwds={}):
        QWidgetMock.__init__(self, *baseargs, **basekwds)
        self.path_edit = QLineEditMock()

    def set_path(self, path):
        self.path_edit.setText(path)

class NumericalLineEditMock(QLineEditMock):
    _MockRealClass = widgets.NumericalLineEdit



""" Test the signal module. """
import os.path, time, signal

import srllib.process as _process
from srllib import util
from _common import *

class TestError(_process.PickleableException):
    pass

def _childfunc_writes(outstr, errstr):
    for i in range(10):
        sys.stdout.write("%s\n" % outstr)
        sys.stderr.write("%s\n" % errstr)
    
def _childfunc_raises():
    raise TestError("TestError")

def _childfunc_sleeps():
    import time
    while True:
        time.sleep(0.1)

class ProcessTest(TestCase):
    def test_child_exception(self):
        """ Test catching an exception raised in the child. """
        proc = _process.Process(_childfunc_raises)
        try:
            try: proc.wait()
            except _process.ChildError, err:
                self.assert_(isinstance(err.orig_exception, TestError))
            else:
                print proc.stderr.read()
                raise AssertionError, "Exception not raised"
            # Polling should raise the same error
            self.assertRaises(_process.ChildError, proc.poll)
        finally:
            proc.close()
        
    def test_terminate(self):
        """ Test terminating the child process. """
        if util.get_os_name() == util.Os_Windows:
            # This method is only usable if pywin32 is installed
            try: import win32process
            except ImportError: return

        proc = _process.Process(_childfunc_sleeps)
        self.assertEqual(proc.terminate(), -signal.SIGTERM)
        # Make sure that it is safe to call this after the process has exited
        self.assertEqual(proc.terminate(), -signal.SIGTERM)
        proc.close()
    
    '''
    def test_run_in_terminal(self):
        """ Test running code in virtual terminal. """
        if get_os_name() == Os_Linux:
            # Execute child function under supervision and observe outputs
            def slot_stdout(txt):
                self.__stdout += txt
            def slot_stderr(txt):
                self.__stderr += txt
            
            self.__stdout, self.__stderr = "", ""
            procmon = _process.ThreadedProcessMonitor(use_pty=True)
            self._connect_to(procmon.sig_stdout, slot_stdout)
            self._connect_to(procmon.sig_stderr, slot_stderr)
            
            procmon(_childfunc_writes, ["Test out", "Test err"])
            procmon.wait()

            for l in self.__stdout.splitlines():
                self.assertEqual(l, "Test out")
            for l in self.__stderr.splitlines():
                self.assertEqual(l, "Test err")
    '''
       
def _childfunc_succeeds():
    pass  

def _childfunc_fails():
    raise Exception("Failure")     
                
class ThreadedProcessMonitorTest(TestCase):
    """ Test the threaded process monitor. """
    '''
    def test_capture_output(self):
        """ Test capturing textual output from child process. """
        def slot_stdout(text):
            self.__stdout += text
        def slot_stderr(text):
            self.__stderr += text
            
        def childfunc(process):
            import sys
            sys.stdout.write("Test stdout")
            sys.stderr.write("Test stderr"    )
            
        procmon = _process.ThreadedProcessMonitor()
        self.__stdout, self.__stderr = "", ""
        self._connect_to(procmon.sig_stdout, slot_stdout)
        self._connect_to(procmon.sig_stderr, slot_stderr)
        procmon(childfunc)
        procmon.wait()
        self.assertEqual(self.__stdout, "Test stdout")
        self.assertEqual(self.__stderr, "Test stderr")
    '''
    
    def test_success(self):
        """ Verify that sig_finished is received when the process finishes
        successfully. """
        def slot_finished():
            self.__finished = True
        def slot_failed(err):
            self.__failed = True
        
        procmon = _process.ThreadedProcessMonitor()
        self.__finished = self.__failed = False
        self._connect_to(procmon.sig_finished, slot_finished)
        self._connect_to(procmon.sig_failed, slot_failed)
        procmon(_childfunc_succeeds)
        procmon.wait()
        self.assert_(self.__finished)
        self.assertNot(self.__failed)
        
    def test_failure(self):
        """ Verify that sig_failed is received when the process fails. """
        def slot_finished():
            self.__finished = True
        def slot_failed(err):
            self.__error = err
        
        procmon = _process.ThreadedProcessMonitor()
        self.__finished = False
        self.__error = None
        self._connect_to(procmon.sig_finished, slot_finished)
        self._connect_to(procmon.sig_failed, slot_failed)
        procmon(_childfunc_fails)
        procmon.wait()
        self.assert_(isinstance(self.__error, _process.ChildError), self.__error)
        self.assertNot(self.__finished)
        
    def test_command(self):
        """ Test monitoring a command (invoke a command instead of a Python
        callable).
        """
        procmon = _process.ThreadedProcessMonitor()
        # Use realpath in case we get a path with symlink(s)
        cwd = os.path.realpath(self._get_tempdir())
        prcs = procmon.monitor_command(["python", "-c", "import os; print "
                "os.getcwd()"], cwd=cwd)
        
        self.assertEqual(prcs.stdout.read().strip(), cwd)
    



""" Object inspection utilities.
"""
def _getattr_recursive(obj, attr, include_bases):
    try: return getattr(obj, attr)
    except AttributeError:
        if include_bases:
            for cls in obj.__bases__:
                try: return _getattr_recursive(cls, attr, include_bases)
                except AttributeError: continue

def get_members(obj, predicate=None, include_bases=True):
    """ Replacement for inspect.getmembers.
    @param predicate: If specified, a function which takes a class attribute and
    indicates (True/False) whether or not it should be included.
    @return: Dictionary of members.
    """
    mems = {}
    for attr in dir(obj):
        val = _getattr_recursive(obj, attr, include_bases)
        if predicate is None or predicate(val):
            mems[attr] = val
    return mems


""" Functionality for managing child processes. """
# Don't import signal from this package
from __future__ import absolute_import
import os.path, struct, cPickle, sys, signal, traceback, subprocess, errno, \
    stat, time

from srllib import threading, util
from srllib._common import *
from srllib.error import BusyError, SrlError
         
class ChildError(SrlError):
    """ Exception detected in child process.
    
    If the original exception derives from L{PickleableException}, it is
    preserved, along with the traceback.
    @ivar orig_exception: The original exception, possibly C{None}.
    @ivar orig_traceback: Traceback of original exception, possibly C{None}.
    """
    def __init__(self, process_error):
        SrlError.__init__(self, "Error in child process")    
        if process_error.exc_class is not None:
            assert process_error.exc_message is not None 
            assert process_error.exc_traceback is not None
            assert process_error.exc_arguments is not None
            self.orig_exception = process_error.exc_class(
                *process_error.exc_arguments)
        else:
            self.orig_exception = None
        self.orig_traceback = process_error.exc_traceback
                    
class PickleableException(SrlError):
    def __init__(self, msg, *args):
        SrlError.__init__(self, msg)
        self.arguments = (msg,) + args

class _ProcessError(object):
    """ Encapsulation of a child process error.
    
    Exceptions don't pickle in the standard fashion, so we do it like this.
    @ivar message: Error message.
    @ivar exc_message: Original exception message.
    @ivar exc_class: Class of original exception.
    @ivar exc_arguments: Arguments of original exception.
    @ivar exc_traceback: Traceback of original exception.
    """
    def __init__(self, msg, original_exc=None, original_tb=None):
        self.message = msg
        
        if original_exc is not None:            
            self.exc_message = str(original_exc)
        else:
            self.exc_message = None
        if isinstance(original_exc, PickleableException):
            self.exc_class = original_exc.__class__
            self.exc_arguments = original_exc.arguments
        else:
            self.exc_class = None
            self.exc_arguments = None
        if original_tb is not None:
            self.exc_traceback = traceback.format_tb(original_tb)
        else:
            self.exc_traceback = None
            
class PickleError(SrlError):
    pass

class _MthdProxy(object):
    def __init__(self, mthd):
        self.__obj, self.__cls, self.__name = (mthd.im_self, mthd.im_class,
            mthd.im_func.func_name)

    def __call__(self, *args, **kwds):
        name = self.__name
        if name.startswith("__"):
            # Mangle
            name = "_%s%s" % (self.__cls.__name__, name)
        func = getattr(self.__cls, name)
        func(self.__obj, *args, **kwds)

class ChildDied(SrlError):
    """ Child died unexpectedly.
    @ivar exitcode: Child's exit code.
    @ivar stderr: Child's stderr.
    """
    def __init__(self, exitcode, stderr):
        SrlError.__init__(self, "Child died unexpectedly (exit code %d)" %
                          exitcode)
        self.exitcode, self.stderrr = exitcode, stderr
        
def terminate(process):
    """ Terminate a process of either the L{Process} type or the standard
    subprocess.Popen type.
    
    This method will block until it is determined that the process has in fact
    terminated.
    @note: On Windows, pywin32 is required.
    @return: The process's exit status.
    """
    r = process.poll()
    if r is not None:
        return r
    if get_os_name() == Os_Windows:
        import win32process, win32api
        # Emulate POSIX behaviour, where the exit code will be the negative
        # value of the signal that terminated the process
        # Open with rights to terminate and synchronize
        handle = win32api.OpenProcess(0x1 | 0x100000, False, process.pid)
        win32process.TerminateProcess(handle, -signal.SIGTERM)
    else:
        try:
            os.kill(process.pid, signal.SIGTERM)
            time.sleep(.05)
        except OSError, err:
            if err.errno == errno.ECHILD:
                # Presumably, the child is dead already?
                pass
            else:
                raise
        if process.poll() is None:
            os.kill(process.pid, signal.SIGKILL)

    return process.wait()
    
class Process(object):
    """ Invoke a callable in a child process.

    Instantiating an object of this class will spawn a child process, I{in which a
    provided callable is invoked}. Pipes are provided for the standard streams
    and a separate pipe for communication between parent and child. stdout and
    stderr are made non-blocking so they can easily be drained of data.
    @ivar stdout: Child's stdout file.
    @ivar stderr: Child's stderr file..
    """
    def __init__(self, child_func, child_args=[], child_kwds={}):
        """
        @param child_func: Function to be called in child process.
        @param child_args: Optional arguments for the child function.
        @param child_kwds: Optional keywords for the child function.
        @raise ChildDied: Child died unexpectedly.
        """
        self.__exit_rslt = None
        # We execute in the child a script which first unpickles the function
        # and its parameters, and then invokes it. This is the best cross-
        # platform approach that we've found (since Windows does not support
        # fork)
        script_fname = self.__script_fname = util.create_tempfile(content=
r"""import cPickle, sys, struct
pipe_path = sys.argv[1]
lnth = struct.unpack("@I", sys.stdin.read(4))[0]
sys.path = cPickle.loads(sys.stdin.read(lnth))
lnth = struct.unpack("@I", sys.stdin.read(4))[0]
func, args, kwds = cPickle.loads(sys.stdin.read(lnth))
try: func(*args, **kwds)
except Exception, err:
    from srllib.process import _ProcessError
    pickle = cPickle.dumps(_ProcessError("Error in child", err, sys.exc_info()[2]))
    f = file(pipe_path, "wb")
    try: f.write(pickle)
    finally: f.close()
""")
        
        # XXX: Some day we might want to use a real (named) pipe ...
        pipe_path = self.__errpipe_path = util.create_tempfile()
        prcs = self.__prcs = subprocess.Popen(["python", script_fname, pipe_path],
                                    stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE, universal_newlines=
                                    True, bufsize=-1)
        
        import types
        if isinstance(child_func, types.MethodType):
            # Can't pickle instance methods
            child_func = _MthdProxy(child_func)
        # Pickle the path, to ensure proper unpickling
        path_data = cPickle.dumps(sys.path)
        try: func_data = cPickle.dumps((child_func, child_args, child_kwds))
        except TypeError, err:
            print err
            raise PickleError("Failed to pickle %r, is this e.g. a nested definition?" % \
                    child_func)
        try:
            prcs.stdin.write(struct.pack("@I", len(path_data)))
            prcs.stdin.write(path_data)
            prcs.stdin.write(struct.pack("@I", len(func_data)))
            prcs.stdin.write(func_data)
            # Important: Flush what we've written so child isn't stuck waiting for data
            prcs.stdin.flush()
        except EnvironmentError, err:
            if err.errno == errno.EPIPE:
                exitcode = self.wait()
                raise ChildDied(exitcode, prcs.stderr.read())
            raise

        self._pid = prcs.pid

    def __str__(self):
        return self.name

    @property
    def pid(self):
        return self._pid
    
    @property
    def stdout(self):
        return self.__prcs.stdout
    
    @property
    def stderr(self):
        return self.__prcs.stderr

    def close(self):
        """ Release resources.

        If the process is still alive, it is waited for.
        """
        if self.__exit_rslt is None:
            self.wait()
        os.remove(self.__errpipe_path)
        os.remove(self.__script_fname)
            
    def poll(self):
        """ Check if child has exited.
        @return: If child has exited, its exit code, else C{None}.
        """
        rslt = self.__exit_rslt
        if rslt is not None:
            if isinstance(rslt, ChildError):
                raise rslt 
            assert isinstance(rslt, int)
            return rslt
        r = self.__prcs.poll()
        f = file(self.__errpipe_path, "rb")
        try: data = f.read()
        finally: f.close()
        if data:
            # Exception from child process
            err = cPickle.loads(data)
            self.__exit_rslt = ChildError(err)            
            raise self.__exit_rslt
        return r

    def wait(self):
        """ Wait for child to finish.
        @return: Child's exit code.
        @raise ChildError: Exception detected in child.
        """
        self.__prcs.wait()
        return self.poll()

    def terminate(self):
        """ Kill child process.
        
        Implemented using L{terminate}.
        """
        return terminate(self)

    def write_message(self, message, wait=True):
        """ Write message to other process.
        
        If this is the child process, message will be available for parent
        process and vice versa. This method may wait for the other process to
        "pick up the phone". A broken connection will result in EofError.
        @param message: An arbitrary object.
        @param wait: Wait for acknowledgement.
        """
        msg = cPickle.dumps(message)
        self.pipe_out.write(struct.pack("i", len(msg)))
        self.pipe_out.write(msg)
        if not wait:
            return
        # Wait for ack
        a = self.pipe_in.read(1)
        if a == "":
            raise EofError

    def read_message(self):
        """ Read message from other process.
        
        If this is the child process, message will be read from parent process
        and vice versa. This method will wait until a message is actually
        received.
        @return: An arbitrary object written by the other process
        @raise EofError: Broken connection.
        """
        def read_data(lnth):
            data = self.pipe_in.read(lnth)
            if len(data) < lnth:
                raise EofError
            return data
        
        data = read_data(struct.calcsize("i"))
        msgLnth = struct.unpack("i", data)[0]
        data = read_data(msgLnth)

        # Ack
        try: self.pipe_out.write('a')
        except IOError: pass

        import cPickle
        obj = cPickle.loads(data)
        return obj

    def _poll(self, wait=False):
        if self._childRet is not None:
            return self._childRet

        if wait:
            flag = 0
        else:
            flag = os.WNOHANG
        while True:
            try: pid, status = os.waitpid(self._pid, flag)
            except OSError, err:
                # Ignore interrupts
                if err.errno == errno.EINTR:
                    continue
                raise
            break

        if pid != self._pid:
            return None

        if os.WIFSIGNALED(status):
            self._childRet = -os.WTERMSIG(status)
        else:
            self._childRet = os.WEXITSTATUS(status)

        if self._childRet != 0:
            try: obj = self.read_message()
            except EofError:
                pass
            else:
                if isinstance(obj, _ProcessError):
                    raise ChildError(obj)
        return self._childRet


class EofError(IOError):
    pass

class ThreadedProcessMonitor(object):
    """ Monitor a child process in a background thread.
    @group Signals: sig*
    @ivar process: The L{child process<Process>}
    @ivar sig_stdout: Triggered to deliver stdout output from the child process.
    @ivar sig_stderr: Triggered to deliver stderr output from the child process-
    @ivar sig_finished: Signal that monitor has finished, from background thread.
    Parameters: None.
    @ivar sig_failed: Signal that monitored process failed, from background
    thread. Paramaters: The caught exception.
    """
    def __init__(self, daemon=False, use_pty=False, pass_process=True):
        """
        @param daemon: Start background threads in daemon mode
        @param use_pty: Open pseudo-terminal for child process.
        @param pass_process: When executing functions in child processes,
        should the L{Process} object be passed as a parameter?
        """
        self.sig_stdout, self.sig_stderr, self.sig_finished, self.sig_failed = (
                Signal(), Signal(), Signal(), Signal())
        self.__process = None
        i, o = os.pipe()
        self._event_pipe_in, self._event_pipe_out = (os.fdopen(i, "r", 0),
            os.fdopen(o, "w", 0))
        self._daemon = daemon
        self._thrd = None

    @property
    def process(self):
        """ The monitored process. """
        return self.__process

    def __call__(self, child_func, child_args=[], child_kwds={}):
        """ Execute function in child process, monitored in background thread.
        @param child_func: Function to execute
        @param child_args: Arguments for child function
        @param child_kwds: Keywords for child function
        @raise BusyError: Already busy with a child process.
        """
        if self.__process is not None:
            raise BusyError("Another process is already being monitored")
        self.__exit_code = None
        self.__process = Process(child_func, child_args=child_args, child_kwds=
            child_kwds)
        thrd = self._thrd = threading.Thread(target=self._thrdfunc, daemon=
            self._daemon)
        thrd.start()
        
    def monitor_command(self, arguments, cwd=None, env=None):
        """ Monitor a command.

        @return: The associated L{process<subprocess.Popen>}.
        """
        if self.__process is not None:
            raise BusyError("Another process is already being monitored")
        prcs = self.__process = subprocess.Popen(arguments, cwd=cwd, env=env,
                                          stdout=subprocess.PIPE, stderr=
                                          subprocess.PIPE)
        thrd = self._thrd = threading.Thread(target=self._thrdfunc, daemon=
            self._daemon)
        thrd.start()

        return prcs

    def wait(self):
        """ Wait for monitoring thread to finish.
        @return: The child process exit code. If the child raised a L{ChildError},
        this will be None.
        """
        if self._thrd is not None:
            self._thrd.join()

        return self.__exit_code

    def _thrdfunc(self):
        prcs = self.__process
        try: self.__exit_code = prcs.wait()
        except ChildError, err:
            self.sig_failed(err)
        else:
            self.sig_finished()
        if hasattr(prcs, "close"):
            prcs.close()
        self.__process = None

    '''
    # POSIX
    def _thrdfunc(self):
        import select
        process = self.__process
        stdout, stderr = process.stdout, process.stderr
        pollIn, pollOut, pollEx = [stdout, stderr, self._event_pipe_in], [], []
        while pollIn:
            # Keep in mind that closed files will be seen as ready by select and
            # cause it to wake up
            rd, wr, ex = select.select(pollIn, pollOut, pollEx, 0.01)

            if stdout in rd:
                try: o = stdout.read()
                except IOError:
                    # Presumably EOF
                    o = ""
                if o:
                    self.sig_stdout(o)
                else:
                    pollIn.remove(stdout)
            if stderr in rd:
                e = stderr.read()
                if e:
                    self.sig_stderr(e)
                else:
                    pollIn.remove(stderr)

            if self._event_pipe_in in rd:
                event = self._event_pipe_in.read(1)
                assert event in ("t",)
                if event == "t":
                    # Terminate
                    process.terminate()
                    break

            if stdout not in pollIn and stderr not in pollIn:
                # Child exited
                break

        try: self.__exit_code = self.__process.wait()
        except ChildError, err:
            self.sig_failed(err)
        else:
            self.sig_finished()
        self.__process.close()
        self.__process = None
    '''



""" Test util module. """
from testqtgui._common import *

from srllib.qtgui import util

class _TestCommand(QtGui.QUndoCommand):
    def __init__(self):
        QtGui.QUndoCommand.__init__(self)
        self.done = False

    def redo(self):
        self.done = True

    def undo(self):
        pass

class UndoStackTest(TestCase):
    def test_construct(self):
        stack = util.UndoStack()
        self.assert_(stack.is_enabled)

    def test_construct_disabled(self):
        stack = util.UndoStack(enable=False)
        self.assertNot(stack.is_enabled)

    def test_push(self):
        stack = util.UndoStack()
        cmd = _TestCommand()
        stack.push(cmd)
        self.assert_(cmd.done)

    def test_disable(self):
        """ Test disabling stack. """
        stack = util.UndoStack()
        stack.is_enabled = False
        self.__test_push(stack)

    def test_enable(self):
        """ Test enabling stack. """
        stack = util.UndoStack(enable=False)
        stack.is_enabled = True
        self.__test_push(stack)

    def __test_push(self, stack):
        count = stack.count()
        cmd = _TestCommand()
        stack.push(cmd)
        self.assert_(cmd.done)
        if stack.is_enabled:
            self.assertEqual(stack.count(), count+1)
        else:
            self.assertEqual(stack.count(), count)

class VariousTest(QtTestCase):
    def test_Action(self):
        """ Test Action factory. """
        def slot():
            self.__called = True
        self._set_attr(QtGui, "QAction", mock.MockFactory(guimocks.QActionMock))
        self._set_attr(QtGui, "QIcon", mock.MockFactory(guimocks.QIconMock))
        self.__called = False

        action = util.Action("action", slot, "icon", "ctrl+s", None)
        icon = guimocks.QIconMock.mockGetAllInstances()[0]
        self.assertEqual(action.mockConstructorArgs, (icon, "action", None))
        action.mockCheckNamedCall(self, "setShortcut", 0, "ctrl+s")
        action.trigger()
        self.assert_(self.__called)


class _BrowseFile(util._BrowseHelper, guimocks.QWidgetMock):
    def __init__(self, *args, **kwds):
        guimocks.QWidgetMock.__init__(self)
        self.path_edit = guimocks.QLineEditMock()
        util._BrowseHelper.__init__(self, *args, **kwds)


class BrowseHelperTest(QtTestCase):
    def test_construct(self):
        browse = self.__test_construct()
        self.assertNot(browse.path_edit.isReadOnly())
        browse = self.__test_construct(kwds={"readonly": True})
        self.assert_(browse.path_edit.isReadOnly())
        browse = self.__test_construct(kwds={"path": "file"})

    def __test_construct(self, kwds={}):
        browse = self.__construct(kwds=kwds)
        path = kwds.get("path")

        path_edit = browse.path_edit
        if path is not None:
            self.assertEqual(path_edit.text(), path)
        else:
            self.assertEqual(path_edit.text(), "")
        tooltip = kwds.get("tooltip")
        if tooltip is not None:
            path_edit.mockCheckNamedCall(self, "setToolTip", -1, tooltip)
        return browse

    def __construct(self, kwds={}):
        browser = _BrowseFile(**kwds)
        return browser



from srllib.util import get_os, get_os_name, get_os_version, Os_Linux, Os_Windows, OsCollection_Posix
from srllib.testing import *


def only_posix(func):
    """Decorator for tests that are particular to POSIX."""
    if get_os_name() in OsCollection_Posix:
        return func
    return None

def only_windows(func):
    """Decorator for tests that are particular to Windows."""
    if get_os_name() == Os_Windows:
        return func
    return None



""" Test the mock module. """
from srllib.testing.mock import Mock, MockInterfaceError

from test_testing._common import *

class _Mocked(object):
    @property
    def prop(self):
        return "prop"

class Mocker(Mock):
    _MockRealClass = _Mocked
    
    @property
    def prop(self):
        pass

class InterfaceTest(TestCase):
    """ Verify interface checking of Mock.
    """
    def test_properties_invalid(self):
        """ Verify that an invalid property is caught.
        """
        class Mocker(Mock):
            _MockRealClass = _Mocked
            
            @property
            def prop(self):
                pass
            
            @property
            def invalidprop(self):
                pass
            
        self.assertRaises(MockInterfaceError, Mocker)
    
    def test_properties_valid(self):
        """ Verify that valid properties pass.
        """ 
        Mocker()
    



""" Exception classes. """
class SrlError(Exception):
    """ Base SRL exception. """

class BusyError(SrlError):
    """ General indication that an object is busy with an operation. """

class NotFound(SrlError):
    """ General indication that a resource was not found. """
    
class Canceled(SrlError):
    """ The operation was canceled. """



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from srllib.util import (get_os, get_os_name, get_os_version, Os_Linux,
    Os_Windows)
from srllib.testing.qtgui import *
from srllib.testing.qtgui import mock as guimock
from srllib.testing.qtgui import mocks as guimocks



from zope.interface import Interface

from srllib.testing.mock import Mock
import srllib.inspect

class InterfaceMock(Mock):
    """ Mock with special support for zope.interface.
    """
    def __init__(self, *args, **kwds):
        dontMock = kwds["dontMock"] = []
        for name in srllib.inspect.get_members(Interface, callable,
            include_bases=False):
            dontMock.append(name)
        Mock.__init__(self, *args, **kwds)


""" Threading logic.

The functionality here improves upon that in the standard L{threading} module.
"""
from __future__ import absolute_import
import threading, os.path, traceback

from srllib import util
from srllib.error import *

class ThreadError(SrlError):
    """ Encapsulation of an exception caught in a thread.
    @ivar name: Thread name.
    @ivar exc_type: Exception type.
    @ivar exc_value: Exception value.
    @ivar exc_traceback: Exception traceback.
    """
    def __init__(self, name, exc_info):
        SrlError.__init__(self, "An exception was caught in thread '%s':\n%s" %
                (name, " ".join(traceback.format_exception(*exc_info))))
        self.name = name
        self.exc_type, self.exc_value, self.exc_traceback = exc_info

class Cancellation(Exception):
    pass

class TimeoutError(Exception):
    pass

def _def_handler(thread_exc):
    print thread_exc
_prev_handler = _exc_handler = _def_handler

def register_exceptionhandler(handler):
    """ Register a handler for exceptions happening in background thread.

    The exception handler will receive a L{ThreadError}. """
    global _exc_handler, _prev_handler
    if _prev_handler != handler:
        _prev_handler = handler
    _exc_handler = handler

def restore_exceptionhandler():
    """ Restore global exception handler to previous one. """
    global _exc_handler, _prev_handler
    _exc_handler = _prev_handler

def synchronized(func):
    """ Decorator for making functions thread-safe. """
    def syncfunc(*args, **kwds):
        func._sync_lock.acquire()
        try: r = func(*args, **kwds)
        finally: func._sync_lock.release()
        return r
    
    func._sync_lock = threading.Lock()
    return syncfunc

_thread_specific = {}

class Lock(object):
    def __init__(self, *args, **kwds):
        self._lk = threading.Lock()
        self.__inError = None

    def acquire(self, *args, **kwds):
        ret = self._lk.acquire(*args, **kwds)
        if ret:
            global _thread_specific
            try:
                locks = _thread_specific[Thread.current_thread()]["locks"]
                locks.append(self)
            except KeyError:
                pass
        if self.__inError:
            raise self.__inError
        return ret

    def release(self, *args, **kwds):
        if self._lk.locked():
            self._lk.release(*args, **kwds)
        global _thread_specific
        try:
            locks = _thread_specific[Thread.current_thread()]["locks"]
            if self in locks:
                locks.remove(self)
        except KeyError:
            pass

    def forceRelease(self, exception=None):
        """ Called by Thread upon in order to forcefully release locks upon exit.
        
        Since held locks are released in an abnormal manner, this will cause the waiting thread to
        receive an exception from acquire().
        """
        self.__inError = exception
        self.release()

class Condition(threading._Condition):
    """ Reimplement threading.Condition in order to provide own Lock implementation as default. This is
    because our own Lock supports forceful release. """
    __super = threading._Condition

    def __init__(self, lock=None):
        if lock is None:
            lock = Lock()
        Condition.__super.__init__(self, lock)
        self.__exc = None

    def wait(self, *args, **kwds):
        Condition.__super.wait(self, *args, **kwds)
        if self.__exc is not None:
            raise self.__exc

    def notifyException(self, exception):
        self.__exc = exception
        self.acquire()
        self.notify()
        self.release()

class Event(object):
    def __init__(self):
        self.__cond = Condition()
        self.__flag = False

    def isSet(self):
        return self.__flag

    def set(self):
        self.__cond.acquire()
        try:
            self.__flag = True
            self.__cond.notifyAll()
        finally:
            self.__cond.release()

    def clear(self):
        self.__cond.acquire()
        try:
            self.__flag = False
        finally:
            self.__cond.release()

    def wait(self, timeout=None):
        self.__cond.acquire()
        try:
            if not self.__flag:
                self.__cond.wait(timeout)
        finally:
            self.__cond.release()

class SynchronousCondition(object):
    """ Synchronize two threads, by having one signal a condition and wait until the other receives
    it. """
    def __init__(self):
        self.__notified, self.__waited = Event(), Event()
        self.__exc = None

    def wait(self):
        """ Wait for condition to become true. """
        if Thread._threadLocal.eventCancel.isSet():
            return
        self.__notified.wait()
        self.__notified.clear()
        self.__waited.set()
        if self.__exc is not None:
            raise self.__exc

    def notify(self):
        """ Signal that condition holds true, wait until other thread gets the message. """
        self.__notified.set()
        if Thread._threadLocal.eventCancel.isSet():
            return
        self.__waited.wait()
        self.__waited.clear()

    def notifyException(self, exception):
        """ Notify waiting threads of exception. """
        self.__exc = exception
        self.__notified.set()
        self.__waited.set()

def test_cancel():
    thrd = Thread.current_thread()
    thrd.test_cancel()

class Thread(object):
    _thread_local = threading.local()
    _thread_local.current = None

    class _DummyThread:
        """ Dummy class for objects that get returned by current_thread if no Thread is controlling the current thread. """
        def __init__(self):
            self.name = "Dummy"
        
        def test_cancel(self):
            pass
    
    def __init__(self, target=None, args=[], kwds={}, name=None, daemon=False, start=False,
                slot_finished=util.no_op):
        """ @param target: function to execute in background thread
        @param args: arguments to target
        @param kwds: keywords to target
        @param name: thread's name
        @param daemon: die with the main thread?
        @param start: start at once?
        @param slot_finished: a function to invoke once the thread finishes
        """
        self._thrd = threading.Thread(target=self._run, name=name)
        self._trgt, self._args, self._kwds = target, args, kwds
        self._slot_finished = slot_finished
        self.__eventCancel = Event()
        global _exc_handler
        self.__exc_handler = _exc_handler

        '''
        for mthd in ("join",):
            setattr(self, mthd, getattr(self._thrd, mthd))
            '''
            
        if daemon:
            self.daemon = True

        if start:
            self.start()

    def __str__(self):
        return self.name

    @classmethod
    def current_thread(cls):
        cur = cls._thread_local.current
        if cur is None:
            return Thread._DummyThread()
        return cur

    def __getName(self):
        return self._thrd.getName()
    def __setName(self, name):
        self._thrd.setName(name)
    name = property(__getName, __setName)

    def __isDaemon(self):
        return self._thrd.isDaemon()
    def __setDaemon(self, daemon):
        self._thrd.setDaemon(daemon)
    daemon = property(__isDaemon, __setDaemon)

    @property
    def alive(self):
        return self._thrd.isAlive()

    def start(self):
        self._thrd.start()

    def join(self):
        self._thrd.join()

    @synchronized
    def cancel(self, wait=False, timeout=None):
        """ Tell this thread to cancel itself. Will wait till the request is honoured.
        It is also possible that the thread finishes its execution independently of this request,
        this function will return anyway when it notices that the thread is no longer running. """
        assert Thread.current_thread() is not self
        self.__eventCancel.set()
        if wait:
            self.join(timeout=timeout)
            if self.alive:
                raise TimeoutError

        self._release_locks()

    def test_cancel(self):
        assert Thread.current_thread() is self
        if self.__eventCancel.isSet():
            
            # Clear event, so that it can be reused
            self.__eventCancel.clear()
            raise Cancellation

    def run(self):
        if self._trgt is None:
            raise NotImplementedError
        self._trgt(*self._args, **self._kwds)

    def register_exception_handler(self, handler):
        """ Set exception handler for this thread. """
        self.__exc_handler = handler

    def unregister_exception_handler(self):
        """ Unset exception handler for this thread. """
        global _exc_handler
        self.__exc_handler = _exc_handler

    def _run(self):
        Thread._thread_local.current = self
        global _thread_specific
        _thread_specific[self] = {"locks": []}
        thrd_exc = None

        try:
            self.run()
        except Cancellation:
            pass
        except:
            import sys
            thrd_exc = ThreadError(self.name, sys.exc_info())
            self.__exc_handler(thrd_exc)
        else:
            self._slot_finished()
        finally:
            # Release all locks held by this thread
            self._release_locks(thrd_exc)
        
    def _release_locks(self, exception=None):
        global _thread_specific
        if not self in _thread_specific:
            return
        for lk in _thread_specific[self]["locks"]:
            lk.forceRelease(exception)
        _thread_specific[self]["locks"] = []



from testqtgui._common import *

import srllib.qtgui.util
from srllib.qtgui import models

class UndoItemModelTest(QtTestCase):
    def test_construct(self):
        model = self.__construct()
        self.assertIs(model.undo_stack, self.__undo_stack)

    def test_construct_without_stack(self):
        """ Test constructing with no undo stack. """
        model = self.__construct(with_undo=False)
        self.assertIsNot(model.undo_stack, None)

    def test_setData(self):
        data = [QtCore.QVariant(x) for x in 1, 2]
        model = self.__construct(hor_headers=("1"), initial_rows=[[data[0]]])
        stack = self.__undo_stack

        model.setData(model.index(0, 0), data[1])
        self.assertEqual(model.data(model.index(0, 0)), data[1])
        self.assertEqual(stack.undoText(), "set item data")
        self.assertEqual(stack.count(), 1)
        stack.undo()
        self.assertEqual(model.data(model.index(0, 0)).toString(),
            data[0].toString())

        # Add data for a role, and verify it isn't touched
        model.setData(model.index(0, 0), QtCore.QVariant(1), Qt.UserRole+1)
        model.setData(model.index(0, 0), QtCore.QVariant(0), Qt.UserRole)
        self.assertEqual(model.data(model.index(0, 0),
            Qt.UserRole+1).toInt()[0], 1)

    def test_setItemData(self):
        def check_data(model, row, column, data):
            for role, var in data.items():
                self.assertEqual(model.data(model.index(row, column),
                    role).toString(), var.toString())

        data = [{}, {}]
        for role in (Qt.EditRole, Qt.UserRole):
            data[0][role] = QtCore.QVariant(0)
            data[1][role] = QtCore.QVariant(1)
        model = self.__construct(hor_headers=("1"), initial_rows=[[data[0]]])
        stack = self.__undo_stack

        model.setItemData(model.index(0, 0), data[1])
        check_data(model, 0, 0, data[1])
        self.assertEqual(stack.undoText(), "set item data")
        self.assertEqual(stack.count(), 1)
        stack.undo()
        check_data(model, 0, 0, data[0])

    def test_setItemData_add(self):
        """ Test adding a role. """
        data = {Qt.UserRole: QtCore.QVariant(0)}
        model = self.__construct(initial_rows=[[data]])
        assert model.itemData(model.index(0, 0)) == data

        add_data = {Qt.UserRole+1: QtCore.QVariant(1)}
        model.setItemData(model.index(0, 0), add_data)
        data.update(add_data)
        self.assertEqual(model.itemData(model.index(0, 0)), data)
        
    def test_setItemData_clear(self):
        """ Test adding a role, clearing others. """
        data = {Qt.UserRole: QtCore.QVariant(0)}
        model = self.__construct(initial_rows=[[data]])

        new_data = {Qt.UserRole+1: QtCore.QVariant(0)}
        model.setItemData(model.index(0, 0), new_data, clear=True)
        self.assertEqual(model.itemData(model.index(0, 0)), new_data)

    def test_appendRow(self):
        class MyItem(QtGui.QStandardItem):
            def clone(self):
                """ Necessary when adding to model. """
                return MyItem(self)

        model = self.__construct(["1"])
        stack = self.__undo_stack

        item = MyItem("text")
        model.appendRow([item])
        self.assertIsNot(model.item(0), item)
        self.assert_(isinstance(model.item(0), MyItem))
        self.assertEqual(stack.count(), 1)
        self.assertEqual(stack.undoText(), "append row")
        stack.undo()
        self.assertEqual(model.rowCount(), 0)
        stack.redo()
        self.assertEqual(model.data(model.index(0, 0)).toString(), "text")
        stack.undo()

        model.appendRow([MyItem("text")], undo_text="add table row")
        self.assertEqual(stack.undoText(), "add table row")

    def test_takeItem(self):
        model = self.__construct(initial_rows=[["text"]])
        stack = self.__undo_stack

        item = model.takeItem(0)
        self.assertEqual(item.text(), "text")
        self.assertIs(model.item(0), None)
        self.assertEqual(stack.count(), 1)
        self.assertEqual(stack.undoText(), "take item")
        stack.undo()
        stack.redo()
        stack.undo()
        self.assertEqual(model.item(0).text(), "text")
        # Now with undo text
        item = model.takeItem(0, undo_text="delete cell")
        self.assertEqual(stack.undoText(), "delete cell")

    def test_setItem(self):
        model = self.__construct(initial_rows=[["old text", "old text"]])
        model.setItem(0, 1, QtGui.QStandardItem("new text"))
        self.assertEqual(model.item(0, 1).text(), "new text")
        self.assertEqual(self.__undo_stack.count(), 0)

    def test_removeRows(self):
        model = self.__construct(initial_rows=[["1"], ["2"], ["3"]])
        stack = self.__undo_stack

        model.removeRows(1, 2)
        self.assertEqual(stack.count(), 1)
        self.assertEqual(model.rowCount(), 1)
        self.assertEqual(model.item(0).text(), "1")
        self.assertEqual(stack.undoText(), "remove rows")
        stack.undo()
        self.assertEqual(model.item(0).text(), "1")
        self.assertEqual(model.item(1).text(), "2")
        self.assertEqual(model.item(2).text(), "3")
        stack.redo()
        self.assertEqual(model.rowCount(), 1)
        stack.undo()
        model.removeRows(0, 3, undo_text="remove table rows")
        self.assertEqual(stack.undoText(), "remove table rows")

    def __construct(self, hor_headers=None, initial_rows=None, with_undo=True):
        if with_undo:
            stack = self.__undo_stack = srllib.qtgui.util.UndoStack()
        else:
            stack = self.__undo_stack = None
        model = models.UndoItemModel(stack, hor_headers=hor_headers)
        if initial_rows:
            model.undo_stack.is_enabled = False
            for row in initial_rows:
                assert isinstance(row, (list, tuple))
                model.append_row(row)
            model.undo_stack.is_enabled = True
        return model
